@@ -140,6 +140,10 @@ class Invalid(IronicException):
     code = http_client.BAD_REQUEST
 
 
+class MalformedRequestURI(Invalid):
+    _msg_fmt = _("Malformed request URI.")
+
+
 class Conflict(IronicException):
     _msg_fmt = _('Conflict.')
     code = http_client.CONFLICT
@@ -525,10 +529,6 @@ class ServiceUnavailable(IronicException):
     _msg_fmt = _("Connection failed")
 
 
-class Forbidden(IronicException):
-    _msg_fmt = _("Requested OpenStack Baremetal API is forbidden")
-
-
 # TODO(dtantsur): leave only one variant
 BadRequest = Invalid
 
@@ -953,21 +953,6 @@ class NodeVerifyFailure(IronicException):
     _msg_fmt = _("Failed to verify node %(node)s: %(reason)s")
 
 
-class ImageRefIsARedirect(IronicException):
-    _msg_fmt = _("Received a URL redirect when attempting to evaluate "
-                 "image reference %(image_ref)s pointing to "
-                 "%(redirect_url)s. This may, or may not be recoverable.")
-    redirect_url = None
-
-    def __init__(self, image_ref=None, redirect_url=None, msg=None):
-        self.redirect_url = redirect_url
-        # Kwargs are expected by IronicException to convert the message.
-        super(ImageRefIsARedirect, self).__init__(
-            message=msg,
-            image_ref=image_ref,
-            redirect_url=redirect_url)
-
-
 class ConcurrentActionLimit(TemporaryFailure):
     # NOTE(TheJulia): We explicitly don't report the concurrent
     # action limit configuration value as a security guard since
@@ -1046,6 +1031,12 @@ class ChildNodeLocked(Conflict):
                  "completed.")
 
 
+class InvalidMetricConfig(Invalid):
+    """Error raised when a metric config is invalid."""
+
+    message = "Invalid value for metrics config option."
+
+
 class MetricsNotSupported(IronicException):
     _msg_fmt = _("Metrics action is not supported. You may need to "
                  "adjust the [metrics] section in ironic.conf.")
@@ -1146,3 +1137,19 @@ class ConfigDriveRegenerationFailure(IronicException):
     """Raised when we fail to handle configuration drive corrections."""
     # NOTE(TheJulia): This is not intended to get raised to a user, but more
     # so we handle known possible failure cases and don't fail horribly.
+
+
+class TraitBasedNetworkingException(IronicException):
+    """Raised when there's an issue with trait based networking feature."""
+    pass
+
+
+class TBNComparatorPrefixMatchTypeMismatch(TraitBasedNetworkingException):
+    """Comparator prefix match can only be used against a string."""
+    pass
+
+
+class TBNAttributeRetrievalException(TraitBasedNetworkingException):
+    """Specified attribute could not be found."""
+    _msg_fmt = _("Could not retrieve attribute %(attr_name)s from "
+                 "passed object")
