@@ -26,13 +26,13 @@ import time
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import fileutils
+from oslo_utils.imageutils import format_inspector as image_format_inspector
 import pycdlib
 
 from ironic.common import checksum_utils
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
 from ironic.common.i18n import _
-from ironic.common import image_format_inspector
 from ironic.common import image_service as service
 from ironic.common import qemu_img
 from ironic.common import utils
@@ -101,9 +101,7 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
              mounting, creating filesystem, copying files, etc.
     """
     try:
-        # TODO(sbaker): use ironic_lib.utils.dd when rootwrap has been removed
-        utils.execute('dd', 'if=/dev/zero', 'of=%s' % output_file, 'count=1',
-                      'bs=%dKiB' % fs_size_kib)
+        utils.dd('/dev/zero', output_file, 'count=1', 'bs=%dKiB' % fs_size_kib)
     except processutils.ProcessExecutionError as e:
         raise exception.ImageCreationFailed(image_type='vfat', error=e)
 
@@ -113,10 +111,7 @@ def create_vfat_image(output_file, files_info=None, parameters=None,
             # The label helps ramdisks to find the partition containing
             # the parameters (by using /dev/disk/by-label/ir-vfd-dev).
             # NOTE: FAT filesystem label can be up to 11 characters long.
-            # TODO(sbaker): use ironic_lib.utils.mkfs when rootwrap has been
-            #              removed
-            utils.execute('mkfs', '-t', 'vfat', '-n',
-                          'ir-vfd-dev', output_file)
+            utils.mkfs('vfat', output_file, 'ir-vfd-dev')
         except processutils.ProcessExecutionError as e:
             raise exception.ImageCreationFailed(image_type='vfat', error=e)
 
@@ -177,7 +172,8 @@ def _label(files_info):
 
 
 def create_isolinux_image_for_bios(
-        output_file, kernel, ramdisk, kernel_params=None, inject_files=None):
+        output_file, kernel, ramdisk, kernel_params=None, inject_files=None,
+        publisher_id=None):
     """Creates an isolinux image on the specified file.
 
     Copies the provided kernel, ramdisk to a directory, generates the isolinux
@@ -193,6 +189,8 @@ def create_isolinux_image_for_bios(
         as the kernel cmdline.
     :param inject_files: Mapping of local source file paths to their location
         on the final ISO image.
+    :param publisher_id: A value to set as the publisher identifier string
+        in the ISO image to be generated.
     :raises: ImageCreationFailed, if image creation failed while copying files
         or while running command to generate iso.
     """
@@ -239,9 +237,12 @@ def create_isolinux_image_for_bios(
         isolinux_cfg = os.path.join(tmpdir, ISOLINUX_CFG)
         utils.write_to_file(isolinux_cfg, cfg)
 
+        # Set a publisher ID value to a string.
+        pub_id = str(publisher_id)
+
         try:
             utils.execute('mkisofs', '-r', '-V', _label(files_info),
-                          '-J', '-l', '-no-emul-boot',
+                          '-J', '-l', '-publisher', pub_id, '-no-emul-boot',
                           '-boot-load-size', '4', '-boot-info-table',
                           '-b', ISOLINUX_BIN, '-o', output_file, tmpdir)
         except processutils.ProcessExecutionError as e:
@@ -251,7 +252,7 @@ def create_isolinux_image_for_bios(
 
 def create_esp_image_for_uefi(
         output_file, kernel, ramdisk, deploy_iso=None, esp_image=None,
-        kernel_params=None, inject_files=None):
+        kernel_params=None, inject_files=None, publisher_id=None):
     """Creates an ESP image on the specified file.
 
     Copies the provided kernel, ramdisk and EFI system partition image (ESP) to
@@ -273,6 +274,8 @@ def create_esp_image_for_uefi(
         as the kernel cmdline.
     :param inject_files: Mapping of local source file paths to their location
         on the final ISO image.
+    :param publisher_id: A value to set as the publisher identifier string
+        in the ISO image to be generated.
     :raises: ImageCreationFailed, if image creation failed while copying files
         or while running command to generate iso.
     """
@@ -339,17 +342,40 @@ def create_esp_image_for_uefi(
         utils.write_to_file(grub_cfg, grub_conf)
 
         # Create the boot_iso.
+        if publisher_id:
+            args = ('mkisofs', '-r', '-V', _label(files_info),
+                    '-l', '-publisher', publisher_id, '-e', e_img_rel_path,
+                    '-no-emul-boot', '-o', output_file,
+                    tmpdir)
+        else:
+            args = ('mkisofs', '-r', '-V', _label(files_info),
+                    '-l', '-e', e_img_rel_path,
+                    '-no-emul-boot', '-o', output_file,
+                    tmpdir)
         try:
-            utils.execute('mkisofs', '-r', '-V', _label(files_info),
-                          '-l', '-e', e_img_rel_path, '-no-emul-boot',
-                          '-o', output_file, tmpdir)
+            utils.execute(*args)
 
         except processutils.ProcessExecutionError as e:
             LOG.exception("Creating ISO image failed.")
             raise exception.ImageCreationFailed(image_type='iso', error=e)
 
 
-def fetch_into(context, image_href, image_file):
+def fetch_into(context, image_href, image_file,
+               image_auth_data=None):
+    """Fetches image file contents into a file.
+
+    :param context: A context object.
+    :param image_href: The Image URL or reference to attempt to retrieve.
+    :param image_file: The file handler or file name to write the requested
+                       file contents to.
+    :param image_auth_data: Optional dictionary for credentials to be conveyed
+                            from the original task to the image download
+                            process, if required.
+    :returns: If a value is returned, that value was validated as the checksum.
+              Otherwise None indicating the process had been completed.
+    """
+    # TODO(TheJulia): We likely need to document all of the exceptions which
+    #                 can be raised by any of the various image services here.
     # TODO(vish): Improve context handling and add owner and auth data
     #             when it is added to glance.  Right now there is no
     #             auth checking in glance, so we assume that access was
@@ -361,6 +387,12 @@ def fetch_into(context, image_href, image_file):
                'image_href': image_href})
     start = time.time()
 
+    if image_service.is_auth_set_needed:
+        # Send a dictionary with username/password data,
+        # but send it in a dictionary since it fundimentally
+        # can differ dramatically by types.
+        image_service.set_image_auth(image_href, image_auth_data)
+
     if isinstance(image_file, str):
         with open(image_file, "wb") as image_file_obj:
             image_service.download(image_href, image_file_obj)
@@ -369,23 +401,107 @@ def fetch_into(context, image_href, image_file):
 
     LOG.debug("Image %(image_href)s downloaded in %(time).2f seconds.",
               {'image_href': image_href, 'time': time.time() - start})
+    if image_service.transfer_verified_checksum:
+        # TODO(TheJulia): The Glance Image service client does a
+        # transfer related check when it retrieves the file. We might want
+        # to shift the model some to do that upfront across most image
+        # services which are able to be used that way.
+
+        # We know, thanks to a value and not an exception, that
+        # we have a checksum which matches the transfer.
+        return image_service.transfer_verified_checksum
+    return None
+
+
+def _handle_zstd_compression(path):
+    zstd_comp = False
+    with open(path, 'rb') as comp_check:
+        # Check for zstd compression. Zstd has a variable window for streaming
+        # clients with transparent connections, and 128 byte blocks.
+        # Ultimately, requests can't support handling of such content without
+        # the zstandard library (bsd), but that is not available in global
+        # requirements. As such, and likely best complexity wise, if we find it
+        # we can handle it directly.
+        # https://github.com/facebook/zstd/blob/dev/doc/zstd_compression_format.md
+        # Ensure we're at the start of the file
+        comp_check.seek(0)
+        read = comp_check.read(4)
+        if read.startswith(b"\x28\xb5\x2f\xfd"):
+            zstd_comp = True
+
+    if zstd_comp and not CONF.conductor.disable_zstandard_decompression:
+        temp_path = path + '.zstd'
+        shutil.move(path, temp_path)
+        try:
+            utils.execute('zstd', '-d', '--rm', temp_path)
+        except OSError as e:
+            LOG.error('Failed to decompress a zstd compressed file: %s', e)
+            # Restore the downloaded file... We might want to fail the
+            # entire process.
+            shutil.move(temp_path, path)
 
 
 def fetch(context, image_href, path, force_raw=False,
-          checksum=None, checksum_algo=None):
+          checksum=None, checksum_algo=None,
+          image_auth_data=None):
     with fileutils.remove_path_on_error(path):
-        fetch_into(context, image_href, path)
-        if (not CONF.conductor.disable_file_checksum
+        transfer_checksum = fetch_into(context, image_href, path,
+                                       image_auth_data)
+        if (not transfer_checksum
+                and not CONF.conductor.disable_file_checksum
                 and checksum):
             checksum_utils.validate_checksum(path, checksum, checksum_algo)
+
+    # Check and decompress zstd files, since python-requests realistically
+    # can't do it for us as-is. Also, some OCI container registry artifacts
+    # may generally just be zstd compressed, regardless if it is a raw file
+    # or a qcow2 file.
+    _handle_zstd_compression(path)
+
     if force_raw:
         image_to_raw(image_href, path, "%s.part" % path)
 
 
+def detect_file_format(path):
+    """Re-implementation of detect_file_format from oslo.utils
+
+    This implementation specifically allows for single multi-format
+    combination of ISO+GPT, which it treats like ISO.
+    """
+
+    with open(path, 'rb') as f:
+        wrapper = image_format_inspector.InspectWrapper(f)
+        try:
+            while f.peek():
+                wrapper.read(4096)
+                if wrapper.formats:
+                    break
+        finally:
+            wrapper.close()
+    try:
+        return wrapper.format
+    except image_format_inspector.ImageFormatError:
+        format_names = set(str(x) for x in wrapper.formats)
+        if format_names == {'iso', 'gpt'}:
+            # If iso+gpt, we choose the iso because bootable-as-block ISOs
+            # can legitimately have a GPT bootloader in front.
+            LOG.debug('Detected %s as ISO+GPT, allowing as ISO', path)
+            return [x for x in wrapper.formats if str(x) == 'iso'][0]
+        # Any other case of multiple formats is an error
+        raise
+
+
 def get_source_format(image_href, path):
     try:
-        img_format = image_format_inspector.detect_file_format(path)
-    except image_format_inspector.ImageFormatError:
+        img_format = detect_file_format(path)
+    except image_format_inspector.ImageFormatError as exc:
+        LOG.error("Parsing of the image %s failed: %s", image_href, exc)
+        raise exception.ImageUnacceptable(
+            reason=_("parsing of the image failed."),
+            image_id=image_href)
+    if img_format is None:
+        LOG.error("Parsing of the image %s failed: format not recognized",
+                  image_href)
         raise exception.ImageUnacceptable(
             reason=_("parsing of the image failed."),
             image_id=image_href)
@@ -395,9 +511,7 @@ def get_source_format(image_href, path):
 def force_raw_will_convert(image_href, path_tmp):
     with fileutils.remove_path_on_error(path_tmp):
         fmt = get_source_format(image_href, path_tmp)
-    if fmt != "raw":
-        return True
-    return False
+    return fmt not in RAW_IMAGE_FORMATS
 
 
 def image_to_raw(image_href, path, path_tmp):
@@ -405,7 +519,7 @@ def image_to_raw(image_href, path, path_tmp):
         if not CONF.conductor.disable_deep_image_inspection:
             fmt = safety_check_image(path_tmp)
 
-            if fmt not in CONF.conductor.permitted_image_formats:
+            if not image_format_permitted(fmt):
                 LOG.error("Security: The requested image %(image_href)s "
                           "is of format image %(format)s and is not in "
                           "the [conductor]permitted_image_formats list.",
@@ -413,14 +527,15 @@ def image_to_raw(image_href, path, path_tmp):
                            'format': fmt})
                 raise exception.InvalidImage()
         else:
-            fmt = get_source_format(image_href, path)
+            fmt = get_source_format(image_href, path_tmp)
             LOG.warning("Security: Image safety checking has been disabled. "
                         "This is unsafe operation. Attempting to continue "
-                        "the detected format %(img_fmt)s for %(path)s.",
+                        "with the detected format %(img_fmt)s for "
+                        "image %(image_href)s.",
                         {'img_fmt': fmt,
-                         'path': path})
+                         'image_href': image_href})
 
-        if fmt != "raw" and fmt != "iso":
+        if fmt not in RAW_IMAGE_FORMATS and fmt != "iso":
             # When the target format is NOT raw, we need to convert it.
             # however, we don't need nor want to do that when we have
             # an ISO image. If we have an ISO because it was requested,
@@ -437,7 +552,7 @@ def image_to_raw(image_href, path, path_tmp):
                                        source_format=fmt)
                 os.unlink(path_tmp)
                 new_fmt = get_source_format(image_href, staged)
-                if new_fmt != "raw":
+                if new_fmt not in RAW_IMAGE_FORMATS:
                     raise exception.ImageConvertFailed(
                         image_id=image_href,
                         reason=_("Converted to raw, but format is "
@@ -448,14 +563,20 @@ def image_to_raw(image_href, path, path_tmp):
             os.rename(path_tmp, path)
 
 
-def image_show(context, image_href, image_service=None):
+def image_show(context, image_href, image_service=None, image_auth_data=None):
     if image_service is None:
         image_service = service.get_image_service(image_href, context=context)
+    if image_service.is_auth_set_needed:
+        # We need to possibly authenticate, so we should attempt to do so.
+        image_service.set_image_auth(image_href, image_auth_data)
     return image_service.show(image_href)
 
 
-def download_size(context, image_href, image_service=None):
-    return image_show(context, image_href, image_service)['size']
+def download_size(context, image_href, image_service=None,
+                  image_auth_data=None):
+    return image_show(context, image_href,
+                      image_service=image_service,
+                      image_auth_data=image_auth_data)['size']
 
 
 def converted_size(path, estimate=False):
@@ -472,7 +593,7 @@ def converted_size(path, estimate=False):
         the original image scaled by the configuration value
         `raw_image_growth_factor`.
     """
-    data = image_format_inspector.detect_file_format(path)
+    data = detect_file_format(path)
     if not estimate:
         return data.virtual_size
     growth_factor = CONF.raw_image_growth_factor
@@ -516,7 +637,7 @@ def get_temp_url_for_glance_image(context, image_uuid):
 def create_boot_iso(context, output_filename, kernel_href,
                     ramdisk_href, deploy_iso_href=None, esp_image_href=None,
                     root_uuid=None, kernel_params=None, boot_mode=None,
-                    inject_files=None):
+                    inject_files=None, publisher_id=None):
     """Creates a bootable ISO image for a node.
 
     Given the hrefs for kernel, ramdisk, root partition's UUID and
@@ -542,6 +663,8 @@ def create_boot_iso(context, output_filename, kernel_href,
     :boot_mode: the boot mode in which the deploy is to happen.
     :param inject_files: Mapping of local source file paths to their location
         on the final ISO image.
+    :param publisher_id: A value to set as the publisher identifier string
+        in the ISO image to be generated.
     :raises: ImageCreationFailed, if creating boot ISO failed.
     """
     with utils.tempdir() as tmpdir:
@@ -570,7 +693,7 @@ def create_boot_iso(context, output_filename, kernel_href,
 
             elif CONF.esp_image:
                 esp_image_path = CONF.esp_image
-            # TODO(TheJulia): we should opportunisticly try to make bios
+            # TODO(TheJulia): we should opportunistically try to make bios
             # bootable and UEFI. In other words, collapse a lot of this
             # path since they are not mutually exclusive.
             # UEFI boot mode, but Network iPXE -> ISO means bios bootable
@@ -578,12 +701,14 @@ def create_boot_iso(context, output_filename, kernel_href,
             create_esp_image_for_uefi(
                 output_filename, kernel_path, ramdisk_path,
                 deploy_iso=deploy_iso_path, esp_image=esp_image_path,
-                kernel_params=params, inject_files=inject_files)
+                kernel_params=params, inject_files=inject_files,
+                publisher_id=publisher_id)
 
         else:
             create_isolinux_image_for_bios(
                 output_filename, kernel_path, ramdisk_path,
-                kernel_params=params, inject_files=inject_files)
+                kernel_params=params, inject_files=inject_files,
+                publisher_id=publisher_id)
 
 
 IMAGE_TYPE_PARTITION = 'partition'
@@ -623,6 +748,11 @@ def is_whole_disk_image(ctx, instance_info):
 
         is_whole_disk_image = (not iproperties.get('kernel_id')
                                and not iproperties.get('ramdisk_id'))
+    elif service.is_container_registry_url(image_source):
+        # NOTE(theJulia): We can safely assume, at least outright,
+        # that all container images are whole disk images, unelss
+        # someone wants to add explicit support.
+        is_whole_disk_image = True
     else:
         # Non glance image ref
         if is_source_a_path(ctx, instance_info.get('image_source')):
@@ -657,24 +787,12 @@ def is_source_a_path(ctx, image_source):
                                               context=ctx)
     try:
         res = image_service.validate_href(image_source)
-        if 'headers' in dir(res):
-            # response/result is from the HTTP check path.
-            headers = res.headers
-        else:
-            # We have no headers.
-            headers = {}
-    except exception.ImageRefIsARedirect as e:
-        # Our exception handling formats this for us in this
-        # case. \o/
-        LOG.debug(str(e))
-        # Servers redirect to a proper folder ending in a slash if
-        # not supplied originally.
-        if e.redirect_url and e.redirect_url.endswith('/'):
-            return True
+        headers = getattr(res, 'headers', {})
     except Exception:
         # NOTE(TheJulia): I don't really like this pattern, *but*
         # the wholedisk image support is similar.
         return
+
     # NOTE(TheJulia): Files should have been caught almost exclusively
     # before with the Content-Length check.
     # When the ISO is mounted and the webserver mount point url is
@@ -700,7 +818,8 @@ def is_source_a_path(ctx, image_source):
         # NOTE(TheJulia): Files on a webserver have a length which is returned
         # when headres are queried.
         return False
-    if image_source.endswith('/'):
+    # In case of redirects, URL may be different from image_source.
+    if res.url.endswith('/'):
         # If all else fails, looks like a URL, and the server didn't give
         # us any hints.
         return True
@@ -815,23 +934,57 @@ def safety_check_image(image_path, node=None):
     """
     id_string = __node_or_image_cache(node)
     try:
-        img_class = image_format_inspector.detect_file_format(image_path)
-        if not img_class.safety_check():
-            LOG.error("Security: The requested image for "
-                      "deployment of node %(node)s fails safety sanity "
-                      "checking.",
+        img_class = detect_file_format(image_path)
+        if img_class is None:
+            LOG.error("Security: The requested user image for the "
+                      "deployment node %(node)s does not match any known "
+                      "format",
                       {'node': id_string})
             raise exception.InvalidImage()
+        img_class.safety_check()
         image_format_name = str(img_class)
-    except image_format_inspector.ImageFormatError:
+    except image_format_inspector.ImageFormatError as exc:
         LOG.error("Security: The requested user image for the "
                   "deployment node %(node)s failed to be able "
-                  "to be parsed by the image format checker.",
-                  {'node': id_string})
+                  "to be parsed by the image format checker: %(exc)s",
+                  {'node': id_string, 'exc': exc})
+        raise exception.InvalidImage()
+    except image_format_inspector.SafetyCheckFailed as exc:
+        LOG.error("Security: The requested image for "
+                  "deployment of node %(node)s fails safety sanity "
+                  "checking: %(exc)s",
+                  {'node': id_string, 'exc': exc})
         raise exception.InvalidImage()
     return image_format_name
 
 
+RAW_IMAGE_FORMATS = {'raw', 'gpt'}  # gpt is a whole-disk image
+
+
+def image_format_permitted(img_format):
+    permitted = set(CONF.conductor.permitted_image_formats)
+    if 'raw' in permitted:
+        permitted.update(RAW_IMAGE_FORMATS)
+    return img_format in permitted
+
+
+def image_format_matches(actual_format, expected_format):
+    if expected_format in ['ari', 'aki']:
+        # In this case, we have an ari or aki, meaning we're pulling
+        # down a kernel/ramdisk, and this is rooted in a misunderstanding.
+        # They should be raw. The detector should be detecting this *as*
+        # raw anyway, so the data just mismatches from a common
+        # misunderstanding, and that is okay in this case as they are not
+        # passed to qemu-img.
+        # TODO(TheJulia): Add a log entry to warn here at some point in
+        # the future as we begin to shift the perception around this.
+        # See: https://bugs.launchpad.net/ironic/+bug/2074090
+        return True
+    if expected_format == 'raw' and actual_format in RAW_IMAGE_FORMATS:
+        return True
+    return expected_format == actual_format
+
+
 def check_if_image_format_is_permitted(img_format,
                                        expected_format=None,
                                        node=None):
@@ -847,25 +1000,15 @@ def check_if_image_format_is_permitted(img_format,
     """
 
     id_string = __node_or_image_cache(node)
-    if img_format not in CONF.conductor.permitted_image_formats:
+    if not image_format_permitted(img_format):
         LOG.error("Security: The requested deploy image for node %(node)s "
                   "is of format image %(format)s and is not in the "
                   "[conductor]permitted_image_formats list.",
                   {'node': id_string,
                    'format': img_format})
         raise exception.InvalidImage()
-    if expected_format is not None and img_format != expected_format:
-        if expected_format in ['ari', 'aki']:
-            # In this case, we have an ari or aki, meaning we're pulling
-            # down a kernel/ramdisk, and this is rooted in a misunderstanding.
-            # They should be raw. The detector should be detecting this *as*
-            # raw anyway, so the data just mismatches from a common
-            # misunderstanding, and that is okay in this case as they are not
-            # passed to qemu-img.
-            # TODO(TheJulia): Add a log entry to warn here at some point in
-            # the future as we begin to shift the perception around this.
-            # See: https://bugs.launchpad.net/ironic/+bug/2074090
-            return
+    if (expected_format is not None
+            and not image_format_matches(img_format, expected_format)):
         LOG.error("Security: The requested deploy image for node %(node)s "
                   "has a format (%(format)s) which does not match the "
                   "expected image format (%(expected)s) based upon "
