@@ -371,6 +371,10 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
             binding_profile['vtep-logical-switch'] = vtep_logical_switch
             binding_profile['vtep-physical-switch'] = vtep_physical_switch
 
+        # Include physical_network if available
+        if ironic_port.physical_network:
+            binding_profile['physical_network'] = ironic_port.physical_network
+
         update_port_attrs['binding:profile'] = binding_profile
 
         if not ironic_port.pxe_enabled:
@@ -412,8 +416,7 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
             is_neutron_iface = node.network_interface == 'neutron'
 
             if is_neutron_iface:
-                binding_fail_fatal = getattr(
-                    CONF.neutron, 'fail_on_port_binding_failure', False)
+                binding_fail_fatal = CONF.neutron.fail_on_port_binding_failure
 
             default_failure_behavior = is_smart_nic or binding_fail_fatal
 
@@ -486,7 +489,8 @@ def remove_ports_from_network(task, network_uuid):
     :param network_uuid: UUID of a neutron network ports will be deleted from.
     :raises: NetworkError
     """
-    add_all_ports = CONF.neutron.add_all_ports
+    pxe_capability = 'pxe_boot' in task.driver.boot.capabilities
+    add_all_ports = CONF.neutron.add_all_ports or not pxe_capability
     if not add_all_ports:
         macs = [p.address for p in task.ports if p.pxe_enabled]
     else:
@@ -658,9 +662,7 @@ def get_neutron_port_data(port_id, vif_id, client=None, context=None,
 
     # Finish setup of bond links configuration injection
     if bond_links:
-        links = []
-        for link in bond_links:
-            links.append(link['id'])
+        links = [link['id'] for link in bond_links]
         # Add the list of links to the bond port
         network_data['links'][0]['bond_links'] = links
         # Add the rest of the links to the metadata.
