@@ -545,7 +545,7 @@ RELEASE_MAPPING = {
             'Deployment': ['1.0'],
             'DeployTemplate': ['1.1'],
             'Port': ['1.11'],
-            'Portgroup': ['1.4'],
+            'Portgroup': ['1.5'],
             'Trait': ['1.0'],
             'TraitList': ['1.0'],
             'VolumeConnector': ['1.0'],
@@ -616,7 +616,51 @@ RELEASE_MAPPING = {
             'FirmwareComponent': ['1.0'],
         }
     },
-    'master': {
+    '23.0': {
+        'api': '1.87',
+        'rpc': '1.58',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.3'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+        }
+    },
+    '23.1': {
+        'api': '1.88',
+        'rpc': '1.58',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.3'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+        }
+    },
+    '24.0': {
         'api': '1.89',
         'rpc': '1.59',
         'objects': {
@@ -638,6 +682,291 @@ RELEASE_MAPPING = {
             'FirmwareComponent': ['1.0'],
         }
     },
+    '24.1': {
+        'api': '1.90',
+        'rpc': '1.59',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.3'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+        }
+    },
+    '25.0': {
+        'api': '1.91',
+        'rpc': '1.59',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+        }
+    },
+    '26.0': {
+        'api': '1.91',
+        'rpc': '1.60',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+        }
+    },
+    '26.1': {
+        'api': '1.93',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.40', '1.39', '1.38', '1.37'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+        }
+    },
+    '27.0': {
+        'api': '1.94',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.41'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+            'InspectionRule': ['1.0'],
+        }
+    },
+    '28.0': {
+        'api': '1.95',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.41'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+            'InspectionRule': ['1.0'],
+        }
+    },
+    '29.0': {
+        'api': '1.96',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.41'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.11'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+            'InspectionRule': ['1.0'],
+        }
+    },
+    '30.0': {
+        'api': '1.99',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.41'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.12'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+            'InspectionRule': ['1.0'],
+        }
+    },
+    '31.0': {
+        'api': '1.99',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.1'],
+            'BIOSSetting': ['1.1'],
+            'Node': ['1.41'],
+            'NodeHistory': ['1.0'],
+            'NodeInventory': ['1.0'],
+            'Conductor': ['1.4'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.12'],
+            'Portgroup': ['1.5'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
+            'Runbook': ['1.0'],
+            'InspectionRule': ['1.0'],
+        }
+    },
+    '32.0': {
+        'api': '1.101',
+        'rpc': '1.61',
+        'objects': {
+            'Allocation': ['1.3', '1.2', '1.1'],
+            'BIOSSetting': ['1.2', '1.1'],
+            'Node': ['1.42', '1.41'],
+            'NodeHistory': ['1.1', '1.0'],
+            'NodeInventory': ['1.1', '1.0'],
+            'Conductor': ['1.6', '1.5', '1.4'],
+            'Chassis': ['1.4', '1.3'],
+            'Deployment': ['1.1', '1.0'],
+            'DeployTemplate': ['1.2', '1.1'],
+            'Port': ['1.15', '1.14', '1.13', '1.12'],
+            'Portgroup': ['1.6', '1.5'],
+            'Trait': ['1.1', '1.0'],
+            'TraitList': ['1.2', '1.1', '1.0'],
+            'VolumeConnector': ['1.1', '1.0'],
+            'VolumeTarget': ['1.1', '1.0'],
+            'FirmwareComponent': ['1.1', '1.0'],
+            'Runbook': ['1.1', '1.0'],
+            'InspectionRule': ['1.1', '1.0'],
+        }
+    },
+    '33.0': {
+        'api': '1.104',
+        'rpc': '1.62',
+        'objects': {
+            'Allocation': ['1.3', '1.2', '1.1'],
+            'BIOSSetting': ['1.2', '1.1'],
+            'Node': ['1.43', '1.42', '1.41'],
+            'NodeHistory': ['1.1', '1.0'],
+            'NodeInventory': ['1.1', '1.0'],
+            'Conductor': ['1.6', '1.5', '1.4'],
+            'Chassis': ['1.4', '1.3'],
+            'Deployment': ['1.1', '1.0'],
+            'DeployTemplate': ['1.2', '1.1'],
+            'Port': ['1.15', '1.14', '1.13', '1.12'],
+            'Portgroup': ['1.8', '1.7', '1.6', '1.5'],
+            'Trait': ['1.1', '1.0'],
+            'TraitList': ['1.2', '1.1', '1.0'],
+            'VolumeConnector': ['1.1', '1.0'],
+            'VolumeTarget': ['1.1', '1.0'],
+            'FirmwareComponent': ['1.1', '1.0'],
+            'Runbook': ['1.1', '1.0'],
+            'InspectionRule': ['1.1', '1.0'],
+        }
+    },
+    # NOTE(TheJulia): Items above this line are for shipped versions
+    # for the mappings. If your making a non-release related change,
+    # make it below. To release, we will preserve a version matching
+    # the release as a separate block of text, like above.
+    'master': {
+        'api': '1.104',
+        'rpc': '1.62',
+        'objects': {
+            'Allocation': ['1.3', '1.2', '1.1'],
+            'BIOSSetting': ['1.2', '1.1'],
+            'Node': ['1.43', '1.42', '1.41'],
+            'NodeHistory': ['1.1', '1.0'],
+            'NodeInventory': ['1.1', '1.0'],
+            'Conductor': ['1.6', '1.5', '1.4'],
+            'Chassis': ['1.4', '1.3'],
+            'Deployment': ['1.1', '1.0'],
+            'DeployTemplate': ['1.2', '1.1'],
+            'Port': ['1.15', '1.14', '1.13', '1.12'],
+            'Portgroup': ['1.8', '1.7', '1.6', '1.5'],
+            'Trait': ['1.1', '1.0'],
+            'TraitList': ['1.2', '1.1', '1.0'],
+            'VolumeConnector': ['1.1', '1.0'],
+            'VolumeTarget': ['1.1', '1.0'],
+            'FirmwareComponent': ['1.1', '1.0'],
+            'Runbook': ['1.1', '1.0'],
+            'InspectionRule': ['1.1', '1.0'],
+        }
+    },
 }
 
 # NOTE(xek): Assign each named release to the appropriate semver.
@@ -655,6 +984,10 @@ RELEASE_MAPPING['yoga'] = RELEASE_MAPPING['20.1']
 RELEASE_MAPPING['zed'] = RELEASE_MAPPING['21.1']
 RELEASE_MAPPING['antelope'] = RELEASE_MAPPING['21.4']
 RELEASE_MAPPING['2023.1'] = RELEASE_MAPPING['21.4']
+RELEASE_MAPPING['2023.2'] = RELEASE_MAPPING['23.0']
+RELEASE_MAPPING['2024.1'] = RELEASE_MAPPING['24.1']
+RELEASE_MAPPING['2024.2'] = RELEASE_MAPPING['26.1']
+RELEASE_MAPPING['2025.1'] = RELEASE_MAPPING['29.0']
 
 # List of available versions with named versions first; 'master' is excluded.
 RELEASE_VERSIONS = sorted(set(RELEASE_MAPPING) - {'master'}, reverse=True)
