@@ -20,10 +20,8 @@ import os
 import secrets
 import time
 
-from openstack.baremetal import configdrive as os_configdrive
 from oslo_config import cfg
 from oslo_log import log
-from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import secretutils
@@ -645,6 +643,8 @@ def wipe_internal_info_on_power_off(node):
     node.del_driver_internal_info('agent_cached_clean_steps')
     # Remove TLS certificate since it's regenerated on each run.
     node.del_driver_internal_info('agent_verify_ca')
+    # Remove agent start attempt tracking since it doesn't survive power off.
+    node.del_driver_internal_info('agent_start_attempted')
 
 
 def wipe_token_and_url(task):
@@ -671,6 +671,8 @@ def wipe_deploy_internal_info(task):
     node.del_driver_internal_info('deploy_step_index')
     node.del_driver_internal_info('steps_validated')
     node.del_driver_internal_info('image_source')
+    # Remove agent start attempt tracking since deployment is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -685,6 +687,8 @@ def wipe_cleaning_internal_info(task):
     node.del_driver_internal_info('cleaning_disable_ramdisk')
     node.del_driver_internal_info('steps_validated')
     node.del_driver_internal_info('declarative_cleaning')
+    # Remove agent start attempt tracking since cleaning is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -698,6 +702,8 @@ def wipe_service_internal_info(task):
     node.del_driver_internal_info('service_disable_ramdisk')
     node.del_driver_internal_info('steps_validated')
     node.del_driver_internal_info('image_source')
+    # Remove agent start attempt tracking since servicing is complete/aborted
+    node.del_driver_internal_info('agent_start_attempted')
     async_steps.remove_node_flags(node)
 
 
@@ -1173,51 +1179,6 @@ def power_state_for_network_configuration(task):
     restore_power_state_if_needed(task, previous)
 
 
-# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
-def build_configdrive(node, configdrive):
-    """Build a configdrive from provided meta_data, network_data and user_data.
-
-    If uuid or name are not provided in the meta_data, they're defaulted to the
-    node's uuid and name accordingly.
-
-    :param node: an Ironic node object.
-    :param configdrive: A configdrive as a dict with keys ``meta_data``,
-        ``network_data``, ``user_data`` and ``vendor_data`` (all optional).
-    :returns: A gzipped and base64 encoded configdrive as a string.
-    """
-    meta_data = configdrive.setdefault('meta_data', {})
-    meta_data.setdefault('uuid', node.uuid)
-    if node.name:
-        meta_data.setdefault('name', node.name)
-
-    user_data = configdrive.get('user_data')
-    if isinstance(user_data, (dict, list)):
-        user_data = jsonutils.dump_as_bytes(user_data)
-    elif user_data:
-        user_data = user_data.encode('utf-8')
-
-    LOG.debug('Building a configdrive for node %s', node.uuid)
-    return os_configdrive.build(meta_data, user_data=user_data,
-                                network_data=configdrive.get('network_data'),
-                                vendor_data=configdrive.get('vendor_data'))
-
-
-# NOTE(TheJulia): Move this to configdrive_utils at some point in the future.
-def get_configdrive_image(node):
-    """Get configdrive as an ISO image or a URL.
-
-    Converts the JSON representation into an image. URLs and raw contents
-    are returned unchanged.
-
-    :param node: an Ironic node object.
-    :returns: A gzipped and base64 encoded configdrive as a string.
-    """
-    configdrive = node.instance_info.get('configdrive')
-    if isinstance(configdrive, dict):
-        configdrive = build_configdrive(node, configdrive)
-    return configdrive
-
-
 def fast_track_able(task):
     """Checks if the operation can be a streamlined deployment sequence.
 
