@@ -17,7 +17,6 @@ import os
 from oslo_config import cfg
 
 from ironic.common.i18n import _
-from ironic.conf import auth
 
 
 VALID_ADD_PORTS_VALUES = {
@@ -46,6 +45,12 @@ opts = [
     cfg.IntOpt('status_check_period', default=60,
                help=_('period (in seconds) to check status of nodes '
                       'on inspection')),
+    cfg.BoolOpt('force_dhcp', default=False,
+                help=_('If True, during managed inspection force the '
+                       'inspection ramdisk to use DHCP on all available '
+                       'interfaces and avoid injecting any static network '
+                       'configuration into the virtual media ISO. This '
+                       'ensures LLDP collection across all interfaces.')),
     cfg.StrOpt('extra_kernel_params', default='',
                help=_('extra kernel parameters to pass to the inspection '
                       'ramdisk when boot is managed by ironic (not '
@@ -56,21 +61,20 @@ opts = [
                        'finishes. Ignored for nodes that have fast '
                        'track mode enabled.')),
     cfg.StrOpt('callback_endpoint_override',
+               deprecated_for_removal=True,
+               deprecated_reason=_('This option was used by inspector '
+                                   'inspect interface, which was removed.'),
                help=_('endpoint to use as a callback for posting back '
                       'introspection data when boot is managed by ironic. '
                       'Standard keystoneauth options are used by default.')),
-    # TODO(dtantsur): change the default to True when ironic-inspector is no
-    # longer supported (and update the help string).
-    cfg.BoolOpt('require_managed_boot', default=None,
+    cfg.BoolOpt('require_managed_boot', default=True,
                 help=_('require that the in-band inspection boot is fully '
                        'managed by the node\'s boot interface. Set this to '
                        'False if your installation has a separate (i)PXE boot '
                        'environment for node discovery or unmanaged '
                        'inspection. You may need to set it to False to '
                        'inspect nodes that are not supported by boot '
-                       'interfaces (e.g. because they don\'t have ports). '
-                       'The default value depends on which inspect interface '
-                       'is used: inspector uses False, agent - True.')),
+                       'interfaces (e.g. because they don\'t have ports).')),
     cfg.StrOpt('add_ports',
                default='pxe',
                help=_('Which MAC addresses to add as ports during '
@@ -88,7 +92,8 @@ opts = [
     cfg.StrOpt('default_hooks',
                default='ramdisk-error,validate-interfaces,ports,architecture',
                help=_('A comma-separated lists of inspection hooks that are '
-                      'run by default. In most cases, the operators will not '
+                      'run by default for the "agent" inspection interface. '
+                      'In most cases, the operators will not '
                       'modify this. The default (somewhat conservative) hooks '
                       'will raise an exception in case the ramdisk reports an '
                       'error, validate interfaces in the inventory, create '
@@ -96,7 +101,8 @@ opts = [
     cfg.StrOpt('hooks',
                default='$default_hooks',
                help=_('Comma-separated list of enabled hooks for processing '
-                      'pipeline. The default for this is $default_hooks. '
+                      'pipeline for the "agent" inspection interface. '
+                      'The default for this is $default_hooks. '
                       'Hooks can be added before or after the defaults '
                       'like this: "prehook,$default_hooks,posthook".')),
     cfg.StrOpt('known_accelerators',
@@ -142,12 +148,56 @@ opts = [
                        'option is used by the "root-device" inspection hook.'))
 ]
 
+discovery_opts = [
+    cfg.BoolOpt('enabled',
+                default=False, mutable=True,
+                help=_("Setting this to True enables automatic enrollment "
+                       "of inspected nodes that are not recognized. "
+                       "When enabling this feature, keep in mind that any "
+                       "machine hitting the inspection callback endpoint "
+                       "will be automatically enrolled. The driver must be "
+                       "set when setting this to True.")),
+    cfg.StrOpt('driver',
+               mutable=True,
+               help=_("The default driver to use for newly enrolled nodes. "
+                      "Must be set when enabling auto-discovery.")),
+    cfg.StrOpt('inspection_scope',
+               default=None,
+               help=_("The default inspection scope for nodes enrolled via "
+                      "auto-discovery.")),
+]
+
+pxe_filter_opts = [
+    cfg.StrOpt('dhcp_hostsdir',
+               help=_('The MAC address cache directory, exposed to dnsmasq.'
+                      'This directory is expected to be in exclusive control '
+                      'of the driver but must be purged by the operator. '
+                      'Required.')),
+    cfg.ListOpt('supported_inspect_interfaces',
+                default=['agent'], mutable=True,
+                help=_("List of inspect interfaces that will be considered "
+                       "by the PXE filter. Only nodes with these interfaces "
+                       "will be enabled.")),
+    cfg.IntOpt('sync_period',
+               default=45, mutable=True,
+               help=_("Period (in seconds) between synchronizing the state "
+                      "of dnsmasq with the database.")),
+]
+
+inspection_rule_opts = [
+    cfg.StrOpt('built_in_rules',
+               mutable=True,
+               help=_("Path to YAML file of built-in inspection rules.")),
+    cfg.StrOpt('mask_secrets',
+               default='always',
+               choices=['always', 'never', 'sensitive'],
+               help=_("Whether to mask secrets in the node information "
+                      "passed to the rules."))
+]
+
 
 def register_opts(conf):
     conf.register_opts(opts, group='inspector')
-    auth.register_auth_opts(conf, 'inspector',
-                            service_type='baremetal-introspection')
-
-
-def list_opts():
-    return auth.add_auth_opts(opts, service_type='baremetal-introspection')
+    conf.register_opts(discovery_opts, group='auto_discovery')
+    conf.register_opts(pxe_filter_opts, group='pxe_filter')
+    conf.register_opts(inspection_rule_opts, group='inspection_rules')
