@@ -43,9 +43,14 @@ opts = [
     cfg.BoolOpt('use_ssl',
                 default=False,
                 help=_('Whether to use TLS for JSON RPC')),
+    cfg.StrOpt('cert_file',
+               help=_("Certificate file the JSON-RPC listener will present "
+                      "to clients when [json_rpc]use_ssl=True.")),
+    cfg.StrOpt('key_file',
+               help=_("Private key file matching cert_file.")),
     cfg.BoolOpt('client_use_ssl',
                 default=False,
-                help=_('Set to True for force TLS connections in the client '
+                help=_('Set to True to force TLS connections in the client '
                        'even if use_ssl is set to False. Only makes sense '
                        'if server-side TLS is provided outside of Ironic '
                        '(e.g. with httpd acting as a reverse proxy).')),
@@ -68,18 +73,39 @@ opts = [
     cfg.Opt('unix_socket_mode', type=Octal(),
             help=_('File mode (an octal number) of the unix socket to '
                    'listen on. Ignored if unix_socket is not set.')),
+    cfg.BoolOpt('debug_log_request_id_only',
+                mutable=True,
+                default=True,
+                help=_('When debug logging is enabled, log only the request '
+                       'ID instead of the full request and response for JSON '
+                       'RPC calls. This reduces log verbosity while still '
+                       'providing some traceability for performance '
+                       'analysis.')),
 ]
 
 
-def register_opts(conf):
-    conf.register_opts(opts, group='json_rpc')
-    keystone.register_auth_opts(conf, 'json_rpc')
-    conf.set_default('timeout', 120, group='json_rpc')
+def register_opts(conf, group='json_rpc'):
+    """Register JSON-RPC options.
+
+    :param conf: The global oslo.config object.
+    :param group: Configuration group name to register options under.
+                  Defaults to 'json_rpc'. Allows reusing the same option
+                  set for different JSON-RPC clients (e.g., networking).
+    """
+    conf.register_opts(opts, group=group)
+    keystone.register_auth_opts(conf, group)
+    conf.set_default('timeout', 120, group=group)
 
 
 def list_opts():
+    """Return options for sample configuration generation."""
     return keystone.add_auth_opts(opts)
 
 
-def auth_strategy():
-    return CONF.json_rpc.auth_strategy or CONF.auth_strategy
+def auth_strategy(group='json_rpc'):
+    """Return effective auth strategy for a given group.
+
+    :param group: Configuration group name. Defaults to 'json_rpc'.
+    :returns: Authentication strategy string.
+    """
+    return getattr(CONF, group).auth_strategy or CONF.auth_strategy
