@@ -24,7 +24,7 @@ from ironic.common.i18n import _
 opts = [
     cfg.StrOpt('kernel_append_params',
                deprecated_name='pxe_append_params',
-               default='nofb nomodeset vga=normal',
+               default='nofb vga=normal',
                mutable=True,
                help=_('Additional append parameters for baremetal PXE boot.')),
     cfg.StrOpt('default_ephemeral_format',
@@ -152,7 +152,7 @@ opts = [
     cfg.IntOpt('ipxe_timeout',
                default=0,
                help=_('Timeout value (in seconds) for downloading an image '
-                      'via iPXE. Defaults to 0 (no timeout)')),
+                      'via iPXE. Timeout is disabled when this value is 0')),
     cfg.IntOpt('boot_retry_timeout',
                min=60,
                help=_('Timeout (in seconds) after which PXE boot should be '
@@ -204,6 +204,11 @@ opts = [
                    'for bootloaders. Use example: '
                    'ipxe.efi:/usr/share/ipxe/ipxe-snponly-x86_64.efi,'
                    'undionly.kpxe:/usr/share/ipxe/undionly.kpxe')),
+    cfg.StrOpt('initial_grub_template',
+               default=os.path.join(
+                   '$pybasedir', 'drivers/modules/initial_grub_cfg.template'),
+               help=_('On ironic-conductor node, the path to the initial grub'
+                      'configuration template for grub network boot.')),
 ]
 
 
