@@ -64,6 +64,7 @@ class Connection(object, metaclass=abc.ABCMeta):
                             nodes with inspection_started_at field before this
                             interval in seconds
                         :instance_uuid: uuid of instance
+                        :instance_name: name of instance
                         :lessee: node's lessee (e.g. project ID)
                         :maintenance: True | False
                         :owner: node's owner (e.g. project ID)
@@ -285,7 +286,8 @@ class Connection(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_port_list(self, limit=None, marker=None,
-                      sort_key=None, sort_dir=None):
+                      sort_key=None, sort_dir=None,
+                      conductor_groups=None, filters=None):
         """Return a list of ports.
 
         :param limit: Maximum number of ports to return.
@@ -294,11 +296,14 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param conductor_groups: A list of conductor groups to filter by,
+                                 defaults to None
+        :param filters: Filters to apply, defaults to None
         """
 
     @abc.abstractmethod
     def get_ports_by_shards(self, shards, limit=None, marker=None,
-                            sort_key=None, sort_dir=None):
+                            sort_key=None, sort_dir=None, filters=None):
         """Return a list of ports contained in the provided shards.
 
         :param shard_ids: A list of shards to filter ports by.
@@ -306,7 +311,7 @@ class Connection(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_ports_by_node_id(self, node_id, limit=None, marker=None,
-                             sort_key=None, sort_dir=None):
+                             sort_key=None, sort_dir=None, filters=None):
         """List all the ports for a given node.
 
         :param node_id: The integer node ID.
@@ -316,12 +321,13 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted
         :param sort_dir: direction in which results should be sorted
                          (asc, desc)
+        :param filters: Filters to apply, defaults to None
         :returns: A list of ports.
         """
 
     @abc.abstractmethod
     def get_ports_by_portgroup_id(self, portgroup_id, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None, filters=None):
         """List all the ports for a given portgroup.
 
         :param portgroup_id: The integer portgroup ID.
@@ -331,6 +337,7 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_key: Attribute by which results should be sorted
         :param sort_dir: Direction in which results should be sorted
                          (asc, desc)
+        :param filters: Filters to apply, defaults to None
         :returns: A list of ports.
         """
 
@@ -397,7 +404,8 @@ class Connection(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_portgroup_list(self, limit=None, marker=None,
                            sort_key=None, sort_dir=None,
-                           project=None):
+                           project=None, conductor_groups=None,
+                           filters=None):
         """Return a list of portgroups.
 
         :param limit: Maximum number of portgroups to return.
@@ -407,6 +415,9 @@ class Connection(object, metaclass=abc.ABCMeta):
         :param sort_dir: Direction in which results should be sorted.
                          (asc, desc)
         :param project: A node owner or lessee to filter by.
+        :param conductor_groups: A list of conductor groups to filter by,
+                                 defaults to None
+        :param filters: Filters to apply, defaults to None
         :returns: A list of portgroups.
         """
 
@@ -584,6 +595,14 @@ class Connection(object, metaclass=abc.ABCMeta):
         :raises: ConductorNotFound
         """
 
+    @abc.abstractmethod
+    def delete_conductor(self, hostname):
+        """Delete a conductor from the database.
+
+        :param hostname: The hostname of this conductor service.
+        :raises: ConductorNotFound if the conductor doesn't exist.
+        """
+
     @abc.abstractmethod
     def touch_conductor(self, hostname, online=True):
         """Mark a conductor as active by updating its 'updated_at' property.
@@ -1556,7 +1575,6 @@ class Connection(object, metaclass=abc.ABCMeta):
         :returns: list of dicts containing shard names and count
         """
 
-    @classmethod
     @abc.abstractmethod
     def create_firmware_component(self, values):
         """Create a FirmwareComponent record for a given node.
@@ -1577,7 +1595,6 @@ class Connection(object, metaclass=abc.ABCMeta):
             records already exists.
         """
 
-    @classmethod
     @abc.abstractmethod
     def update_firmware_component(self, node_id, component, values):
         """Update a FirmwareComponent record.
@@ -1610,7 +1627,6 @@ class Connection(object, metaclass=abc.ABCMeta):
             is not found.
         """
 
-    @classmethod
     @abc.abstractmethod
     def get_firmware_component_list(self, node_id):
         """Retrieve a list Firmware Components of a given node.
@@ -1620,7 +1636,7 @@ class Connection(object, metaclass=abc.ABCMeta):
         :raises: NodeNotFound if the node is not found.
         """
 
-    @abc.abstractclassmethod
+    @abc.abstractmethod
     def get_child_node_ids_by_parent_uuid(self, node_uuid,
                                           exclude_dedicated_power=False):
         """Retrieve a list of child node IDs for a given parent UUID.
@@ -1637,3 +1653,63 @@ class Connection(object, metaclass=abc.ABCMeta):
             should include child nodes with their own power supplies.
         :returns: A list of tuples.
         """
+
+    @abc.abstractmethod
+    def create_inspection_rule(self, values):
+        """Create an inspection rule.
+
+        :param values: A dict describing the rule.
+        :raises: InspectionRuleAlreadyExists if an inspection rule with the
+            same UUID exists.
+        :returns: A rule.
+        """
+
+    @abc.abstractmethod
+    def update_inspection_rule(self, inspection_rule_id, values):
+        """Update an inspection rule.
+
+        :param inspection_rule_id: The id or uuid of an inspection rule.
+        :param values: Dict of values to update.
+        :raises: InspectionRuleNotFound if the rule does not exist.
+        :returns: A rule.
+        """
+
+    @abc.abstractmethod
+    def get_inspection_rule_by_uuid(self, inspection_rule_uuid):
+        """Retrieve an inspection rule by UUID.
+
+        :param inspection_rule_uuid: UUID of the rule to retrieve.
+        :raises: InspectionRuleNotFound if the rule does not exist.
+        :returns: A rule.
+        """
+
+    @abc.abstractmethod
+    def get_inspection_rule_by_id(self, inspection_rule_id):
+        """Retrieve an inspection rule by id.
+
+        :param inspection_rule_id: id of the rule to retrieve.
+        :raises: InspectionRuleNotFound if the rule does not exist.
+        :returns: A rule.
+        """
+
+    @abc.abstractmethod
+    def get_inspection_rule_list(self, limit=None, marker=None, filters=None,
+                                 sort_key=None, sort_dir=None):
+        """Retrieve a list of inspection rules.
+
+        :param limit: Maximum number of rules to return.
+        :param marker: The last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: Direction in which results should be sorted.
+                         (asc, desc)
+        :returns: A list of inspection rules.
+        """
+
+    @abc.abstractmethod
+    def destroy_inspection_rule(self, inspection_rule_id):
+        """Destroy an inspection rule.
+
+        :param inspection_rule_id: ID of the inspection_rule to destroy.
+        :raises: inspection_ruleNotFound if the inspection_rule does not exist.
+        """
