@@ -143,6 +143,7 @@ class NodeBase(Base):
     #             filter on it more efficiently, even though it is
     #             user-settable, and would otherwise be in node.properties.
     instance_uuid = Column(String(36), nullable=True)
+    instance_name = Column(String(255), nullable=True)
     name = Column(String(255), nullable=True)
     chassis_id = Column(Integer, ForeignKey('chassis.id'), nullable=True)
     power_state = Column(String(15), nullable=True)
@@ -269,6 +270,9 @@ class Port(Base):
     physical_network = Column(String(64), nullable=True)
     is_smartnic = Column(Boolean, nullable=True, default=False)
     name = Column(String(255), nullable=True)
+    description = Column(String(255), nullable=True)
+    vendor = Column(String(32), nullable=True)
+    category = Column(String(80), nullable=True)
 
     _node_uuid = orm.relationship(
         "Node",
@@ -299,6 +303,8 @@ class Portgroup(Base):
     standalone_ports_supported = Column(Boolean, default=True)
     mode = Column(String(255))
     properties = Column(db_types.JsonEncodedDict)
+    physical_network = Column(String(64), nullable=True)
+    category = Column(String(80), nullable=True)
 
     _node_uuid = orm.relationship(
         "Node",
@@ -563,6 +569,26 @@ class RunbookStep(Base):
     )
 
 
+class InspectionRule(Base):
+    __tablename__ = 'inspection_rules'
+    __table_args__ = (
+        schema.UniqueConstraint('uuid', name='uniq_inspection_rules0uuid'),
+        Index('inspection_rule_scope_idx', 'scope'),
+        Index('inspection_rule_phase_idx', 'phase'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    uuid = Column(String(36), nullable=False)
+    priority = Column(Integer, nullable=False, default=0)
+    description = Column(String(255), nullable=True)
+    scope = Column(String(255), nullable=True)
+    sensitive = Column(Boolean, default=False)
+    phase = Column(String(16), nullable=True, default='main')
+    conditions = Column(db_types.JsonEncodedList(mysql_as_long=True),
+                        nullable=True)
+    actions = Column(db_types.JsonEncodedList(mysql_as_long=True),
+                     nullable=False)
+
+
 def get_class(model_name):
     """Returns the model class with the specified name.
 
