@@ -14,7 +14,7 @@
 #    under the License.
 
 from oslo_log import log
-from oslo_utils import importutils
+import sushy
 import tenacity
 
 from ironic.common import boot_devices
@@ -22,6 +22,7 @@ from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.common import utils as common_utils
+from ironic.conductor import configdrive_utils
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
@@ -79,8 +80,6 @@ COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
 
 IMAGE_SUBDIR = 'redfish'
 
-sushy = importutils.try_import('sushy')
-
 
 def _parse_driver_info(node):
     """Gets the driver specific Node deployment info.
@@ -124,6 +123,9 @@ def _parse_driver_info(node):
         {option: d_info.get(option, getattr(CONF.conductor, option, None))
          for option in OPTIONAL_PROPERTIES})
 
+    deploy_info['bootloader'] = driver_utils.get_field(
+        node, 'bootloader', use_conf=True)
+
     if (d_info.get('config_via_removable') is None
             and d_info.get('config_via_floppy') is not None):
         LOG.warning('The config_via_floppy driver_info option is deprecated, '
@@ -169,10 +171,87 @@ def _test_retry(exception):
     return False
 
 
-@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
-                stop=tenacity.stop_after_attempt(3),
-                wait=tenacity.wait_fixed(3),
-                reraise=True)
+def _has_vmedia_via_systems(system):
+    """Indicates if virtual media is available through Systems
+
+    :param system: A redfish System object
+    :return: True if the System has virtual media, else False
+    """
+    try:
+        system.virtual_media
+        return True
+    except sushy.exceptions.MissingAttributeError:
+        return False
+    except AttributeError:
+        # NOTE(wncslln): In case of older versions of sushy are
+        # used.
+        return False
+
+
+def _has_vmedia_via_manager(manager):
+    """Indicates if virtual media is available in the Manager
+
+    :param manager: A redfish System object
+    :return: True if the System has virtual media, else False
+    """
+    try:
+        manager.virtual_media
+        return True
+    except sushy.exceptions.MissingAttributeError:
+        return False
+
+
+def _get_vmedia(task, managers):
+    """GET virtual media details
+
+    :param task: A task from Task   Manager.
+    :param managers: A list of System managers.
+    :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+        found on the node.
+    """
+
+    err_msgs = []
+    vmedia_list = []
+    system = redfish_utils.get_system(task.node)
+    if _has_vmedia_via_systems(system):
+        vmedia_get = _get_vmedia_resources(task, system, err_msgs)
+        if vmedia_get:
+            for vmedia in vmedia_get:
+                media_type_list = []
+                for media_type in vmedia.media_types:
+                    media_type_list.append(media_type.value)
+
+                media = {
+                    "media_types": media_type_list,
+                    "inserted": vmedia.inserted,
+                    "image": vmedia.image
+                }
+                vmedia_list.append(media)
+            return vmedia_list
+    else:
+        for manager in managers:
+            vmedia_get = _get_vmedia_resources(task, manager, err_msgs)
+            if vmedia_get:
+                for vmedia in vmedia_get:
+                    media_type_list = []
+                    for media_type in vmedia.media_types:
+                        media_type_list.append(media_type.value)
+
+                    media = {
+                        "media_types": media_type_list,
+                        "inserted": vmedia.inserted,
+                        "image": vmedia.image
+                    }
+                    vmedia_list.append(media)
+                return vmedia_list
+    if err_msgs:
+        exc_msg = ("All virtual media GET attempts failed. "
+                   "Most recent error: ", err_msgs[-1])
+    else:
+        exc_msg = 'No suitable virtual media device found'
+    raise exception.InvalidParameterValue(exc_msg)
+
+
 def _insert_vmedia(task, managers, boot_url, boot_device):
     """Insert bootable ISO image into virtual CD or DVD
 
@@ -184,73 +263,128 @@ def _insert_vmedia(task, managers, boot_url, boot_device):
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    err_msg = None
-    for manager in managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device not in v_media.media_types:
-                # NOTE(janders): this conditional allows v_media that only
-                # support DVD MediaType and NOT CD to also be used.
-                # if v_media.media_types contains sushy.VIRTUAL_MEDIA_DVD
-                # we follow the usual steps of checking if v_media is inserted
-                # and if not, attempt to insert it. Otherwise we skip to the
-                # next v_media device, if any
-                # This is needed to add support to Cisco UCSB and UCSX blades
-                # reference: https://bugs.launchpad.net/ironic/+bug/2031595
-                if (boot_device == sushy.VIRTUAL_MEDIA_CD
-                    and sushy.VIRTUAL_MEDIA_DVD in v_media.media_types):
-                    LOG.debug("While looking for %(requested_device)s virtual "
-                              "media device, found %(available_device)s "
-                              "instead. Attempting to configure it.",
-                              {'requested_device': sushy.VIRTUAL_MEDIA_CD,
-                               'available_device': sushy.VIRTUAL_MEDIA_DVD})
-                else:
-                    continue
-
-            if v_media.inserted:
-                if v_media.image == boot_url:
-                    LOG.debug("Boot media %(boot_url)s is already "
-                              "inserted into %(boot_device)s for node "
-                              "%(node)s", {'node': task.node.uuid,
-                                           'boot_url': boot_url,
-                                           'boot_device': boot_device})
-                    return
-
-                continue
-
-            try:
-                v_media.insert_media(boot_url, inserted=True,
-                                     write_protected=True)
-            # NOTE(janders): On Cisco UCSB and UCSX blades there are several
-            # vMedia devices. Some of those are only meant for internal use
-            # by CIMC vKVM - attempts to InsertMedia into those will result
-            # in BadRequestError. We catch the exception here so that we don't
-            # fail out and try the next available device instead, if available.
-            except sushy.exceptions.BadRequestError:
-                err_msg = ("Inserting virtual media into %(boot_device)s "
-                           "failed for node %(node)s, moving to next virtual "
-                           "media device, if available",
-                           {'node': task.node.uuid,
-                            'boot_device': boot_device})
-                LOG.warning(err_msg)
-                continue
-            except sushy.exceptions.ServerSideError as e:
-                e.node_uuid = task.node.uuid
-                raise
-
-            LOG.info("Inserted boot media %(boot_url)s into "
-                     "%(boot_device)s for node "
-                     "%(node)s", {'node': task.node.uuid,
-                                  'boot_url': boot_url,
-                                  'boot_device': boot_device})
+    err_msgs = []
+    system = redfish_utils.get_system(task.node)
+    if _has_vmedia_via_systems(system):
+        inserted = _insert_vmedia_in_resource(task, system, boot_url,
+                                              boot_device, err_msgs)
+        if inserted:
             return
-    if (err_msg is not None):
+    else:
+        for manager in managers:
+            inserted = _insert_vmedia_in_resource(task, manager, boot_url,
+                                                  boot_device, err_msgs)
+            if inserted:
+                return
+
+    if err_msgs:
         exc_msg = ("All virtual media mount attempts failed. "
-                   "Most recent error: ", err_msg)
+                   "Most recent error: " + err_msgs[-1])
     else:
         exc_msg = 'No suitable virtual media device found'
     raise exception.InvalidParameterValue(exc_msg)
 
 
+@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
+                stop=tenacity.stop_after_attempt(3),
+                wait=tenacity.wait_fixed(3),
+                reraise=True)
+def _insert_vmedia_in_resource(task, resource, boot_url, boot_device,
+                               err_msgs):
+    """Insert virtual media from a given redfish resource (System/Manager)
+
+    :param task: A task from TaskManager.
+    :param resource: A redfish resource either a System or Manager.
+    :param boot_url: URL to a bootable ISO image
+    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`
+    :param err_msgs: A list that will contain all errors found
+    :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+        found on the node.
+    """
+    # Get acceptable virtual media IDs from the boot interface
+    # This allows vendor-specific implementations to restrict which
+    # virtual media devices can be used
+    try:
+        acceptable_id = task.driver.boot._get_acceptable_media_id(task,
+                                                                  resource)
+    except AttributeError:
+        LOG.warning("Boot interface %s does not implement "
+                    "_get_acceptable_media_id method. This method should be "
+                    "implemented for compatibility.",
+                    task.driver.boot.__class__.__name__)
+        acceptable_id = None
+
+    for v_media in resource.virtual_media.get_members():
+        # Skip virtual media if the ID is not in the acceptable set
+        if acceptable_id is not None and v_media.identity != acceptable_id:
+            LOG.debug("Boot Interface %(name)s returned %(acceptable_id)s as "
+                      "Virtual Media Slot ID, current slot is %(slot)s, "
+                      "skipping",
+                      {'name': task.driver.boot.__class__.__name__,
+                       'acceptable_id': acceptable_id,
+                       'slot': v_media.identity})
+            continue
+
+        if boot_device not in v_media.media_types:
+            # NOTE(janders): this conditional allows v_media that only
+            # support DVD MediaType and NOT CD to also be used.
+            # if v_media.media_types contains sushy.VIRTUAL_MEDIA_DVD
+            # we follow the usual steps of checking if v_media is inserted
+            # and if not, attempt to insert it. Otherwise we skip to the
+            # next v_media device, if any
+            # This is needed to add support to Cisco UCSB and UCSX blades
+            # reference: https://bugs.launchpad.net/ironic/+bug/2031595
+            if (boot_device == sushy.VIRTUAL_MEDIA_CD
+                and sushy.VIRTUAL_MEDIA_DVD in v_media.media_types):
+                LOG.debug("While looking for %(requested_device)s virtual "
+                          "media device, found %(available_device)s "
+                          "instead. Attempting to configure it.",
+                          {'requested_device': sushy.VIRTUAL_MEDIA_CD,
+                           'available_device': sushy.VIRTUAL_MEDIA_DVD})
+            else:
+                continue
+
+        if v_media.inserted:
+            if v_media.image == boot_url:
+                LOG.debug("Boot media %(boot_url)s is already "
+                          "inserted into %(boot_device)s for node "
+                          "%(node)s", {'node': task.node.uuid,
+                                       'boot_url': boot_url,
+                                       'boot_device': boot_device})
+                return True
+
+            continue
+
+        try:
+            v_media.insert_media(boot_url, inserted=True,
+                                 write_protected=True)
+        # NOTE(janders): On Cisco UCSB and UCSX blades there are several
+        # vMedia devices. Some of those are only meant for internal use
+        # by CIMC vKVM - attempts to InsertMedia into those will result
+        # in BadRequestError. We catch the exception here so that we don't
+        # fail out and try the next available device instead, if available.
+        except sushy.exceptions.BadRequestError as exc:
+            err_msg = ("Inserting virtual media into %(boot_device)s "
+                       "failed for node %(node)s, moving to next virtual "
+                       "media device, if available. %(exc)s" %
+                       {'node': task.node.uuid, 'exc': exc,
+                        'boot_device': boot_device})
+            err_msgs.append(err_msg)
+            LOG.warning(err_msg)
+            continue
+        except sushy.exceptions.ServerSideError as e:
+            e.node_uuid = task.node.uuid
+            raise
+
+        LOG.info("Inserted boot media %(boot_url)s into "
+                 "%(boot_device)s for node "
+                 "%(node)s", {'node': task.node.uuid,
+                              'boot_url': boot_url,
+                              'boot_device': boot_device})
+        return True
+
+
 def _eject_vmedia(task, managers, boot_device=None):
     """Eject virtual CDs and DVDs
 
@@ -264,40 +398,115 @@ def _eject_vmedia(task, managers, boot_device=None):
         found on the node.
     """
     found = False
+    system = redfish_utils.get_system(task.node)
+    # NOTE(wncslln): we will attempt to eject virtual media in Systems
+    # and in Managers.
+    if _has_vmedia_via_systems(system):
+        ejected = _eject_vmedia_from_resource(task, resource=system,
+                                              boot_device=boot_device)
+        if ejected:
+            found = True
+
     for manager in managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device and boot_device not in v_media.media_types:
-                # NOTE(iurygregory): this conditional allows v_media that only
-                # support DVD MediaType and NOT CD to also be used.
-                # if v_media.media_types contains sushy.VIRTUAL_MEDIA_DVD
-                # we follow the usual steps of checking if v_media is inserted
-                # and eject it. Otherwise we skip to the
-                # next v_media device, if any.
-                # This is needed to add support to Cisco UCSB and UCSX blades
-                # reference: https://bugs.launchpad.net/ironic/+bug/2039042
-                if (boot_device == sushy.VIRTUAL_MEDIA_CD
-                    and sushy.VIRTUAL_MEDIA_DVD in v_media.media_types):
-                    LOG.debug('While looking for %(requested_device)s virtual '
-                              'media device, found %(available_device)s '
-                              'instead. Attempting to use it to eject media.',
-                              {'requested_device': sushy.VIRTUAL_MEDIA_CD,
-                               'available_device': sushy.VIRTUAL_MEDIA_DVD})
-                else:
-                    continue
-            inserted = v_media.inserted
-
-            if inserted:
-                v_media.eject_media()
+        if _has_vmedia_via_manager(manager):
+            ejected = _eject_vmedia_from_resource(task, resource=manager,
+                                                  boot_device=boot_device)
+            if ejected:
                 found = True
+        continue
 
-            LOG.info("Boot media is%(already)s ejected from "
-                     "%(boot_device)s for node %(node)s"
-                     "", {'node': task.node.uuid,
-                          'already': '' if inserted else ' already',
-                          'boot_device': v_media.name})
     return found
 
 
+@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
+                stop=tenacity.stop_after_attempt(3),
+                wait=tenacity.wait_fixed(3),
+                reraise=True)
+def _get_vmedia_resources(task, resource, err_msgs):
+    """Get virtual media for a given redfish resource (System/Manager)
+
+    :param task: A task from TaskManager.
+    :param resource: A redfish resource either a System or Manager.
+    :param err_msgs: A list that will contain all errors found
+    """
+    LOG.info("Get virtual media details for node=%(node)s",
+             {'node': task.node.uuid})
+
+    return resource.virtual_media.get_members()
+
+
+def _eject_vmedia_from_resource(task, resource, boot_device=None):
+    """Eject virtual media from a given redfish resource (System/Manager)
+
+    :param task: A task from TaskManager.
+    :param resource: A redfish resource either a System or Manager.
+    :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
+        eject everything (default).
+    :return: True if any device was ejected, else False
+    """
+    found = False
+    for v_media in resource.virtual_media.get_members():
+        if boot_device and boot_device not in v_media.media_types:
+            # NOTE(iurygregory): this conditional allows v_media that only
+            # support DVD MediaType and NOT CD to also be used.
+            # if v_media.media_types contains sushy.VIRTUAL_MEDIA_DVD
+            # we follow the usual steps of checking if v_media is inserted
+            # and eject it. Otherwise we skip to the
+            # next v_media device, if any.
+            # This is needed to add support to Cisco UCSB and UCSX blades
+            # reference: https://bugs.launchpad.net/ironic/+bug/2039042
+            if (boot_device == sushy.VIRTUAL_MEDIA_CD
+                and sushy.VIRTUAL_MEDIA_DVD in v_media.media_types):
+                LOG.debug('While looking for %(requested_device)s virtual '
+                          'media device, found %(available_device)s '
+                          'instead. Attempting to use it to eject media.',
+                          {'requested_device': sushy.VIRTUAL_MEDIA_CD,
+                           'available_device': sushy.VIRTUAL_MEDIA_DVD})
+            else:
+                continue
+        inserted = v_media.inserted
+        if inserted:
+            v_media.eject_media()
+            found = True
+
+        LOG.info("Boot media is%(already)s ejected from "
+                 "%(boot_device)s for node %(node)s"
+                 "", {'node': task.node.uuid,
+                      'already': '' if inserted else ' already',
+                      'boot_device': v_media.name})
+    return found
+
+
+def get_vmedia(task):
+    """Get the attached virtual CDs and DVDs for a node
+
+        :param task: A task from TaskManager.
+        :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+            found on the node.
+        """
+    LOG.info('Called redfish.boot.get_vmedia, for '
+             'node=%(node)s',
+             {'node': task.node.uuid})
+    system = redfish_utils.get_system(task.node)
+    return _get_vmedia(task, system.managers)
+
+
+def insert_vmedia(task, image_url, device_type):
+    """Insert virtual CDs and DVDs
+
+        :param task: A task from TaskManager.
+        :param image_url:
+        :param device_type: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+            `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
+            eject everything (default).
+        :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+            found on the node.
+        """
+    system = redfish_utils.get_system(task.node)
+    _insert_vmedia(task, system.managers, image_url, device_type)
+
+
 def eject_vmedia(task, boot_device=None):
     """Eject virtual CDs and DVDs
 
@@ -320,8 +529,8 @@ def eject_vmedia(task, boot_device=None):
             image_utils.cleanup_iso_image(task)
 
 
-def _has_vmedia_device(managers, boot_device, inserted=None):
-    """Indicate if device exists at any of the managers
+def _has_vmedia_device(managers, boot_device, inserted=None, system=None):
+    """Indicate if device exists at any of the managers or system
 
     :param managers: A list of System managers.
     :param boot_device: One or more sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
@@ -335,14 +544,23 @@ def _has_vmedia_device(managers, boot_device, inserted=None):
         boot_device = [boot_device]
 
     for dev in boot_device:
-        for manager in managers:
-            for v_media in manager.virtual_media.get_members():
+        if _has_vmedia_via_systems(system):
+            for v_media in system.virtual_media.get_members():
                 if dev not in v_media.media_types:
                     continue
                 if (inserted is not None
                         and bool(v_media.inserted) is not inserted):
                     continue
                 return dev
+        else:
+            for manager in managers:
+                for v_media in manager.virtual_media.get_members():
+                    if dev not in v_media.media_types:
+                        continue
+                    if (inserted is not None
+                            and bool(v_media.inserted) is not inserted):
+                        continue
+                    return dev
     return False
 
 
@@ -401,18 +619,6 @@ class RedfishVirtualMediaBoot(base.BootInterface):
     capabilities = ['iscsi_volume_boot', 'ramdisk_boot',
                     'ramdisk_boot_configdrive']
 
-    def __init__(self):
-        """Initialize the Redfish virtual media boot interface.
-
-        :raises: DriverLoadError if the driver can't be loaded due to
-            missing dependencies
-        """
-        super(RedfishVirtualMediaBoot, self).__init__()
-        if not sushy:
-            raise exception.DriverLoadError(
-                driver='redfish',
-                reason=_('Unable to import the sushy library'))
-
     def get_properties(self):
         """Return the properties of the interface.
 
@@ -478,17 +684,37 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                            if m.manager_type == sushy.MANAGER_TYPE_BMC]
             if bmc_manager:
                 fwv = bmc_manager[0].firmware_version.split('.')
-                if int(fwv[0]) >= 6:
+                if int(fwv[0]) in [6, 7]:
                     return
+            # NOTE(TheJulia) Dell iDRAC10 reverted to 2.x.x.x for versioning
+            # and is also incompatible with the stock virtual media code.
+            # This may be fixed one day, but in the mean time, but the error
+            # provides specific guidance now.
             raise exception.InvalidParameterValue(
                 _("The %(iface)s boot interface is not suitable for node "
                   "%(node)s with vendor %(vendor)s and BMC version %(fwv)s, "
-                  "upgrade to 6.00.00.00 or newer or use "
-                  "idrac-redfish-virtual-media instead")
+                  "the interface only supports iDRAC9 versions between "
+                  "6.0.0.0 and 7.x.x.x. Other versions are incompatible by "
+                  "default and you should use the idrac-redfish-virtual"
+                  "-media boot interface instead")
                 % {'iface': task.node.get_interface('boot'),
                    'node': task.node.uuid, 'vendor': vendor,
                    'fwv': bmc_manager[0].firmware_version})
 
+    def _get_acceptable_media_id(self, task, resource):
+        """Get acceptable virtual media IDs for the resource.
+
+        This method can be overridden by vendor-specific implementations
+        to return specific virtual media IDs that should be used.
+
+        :param task: A TaskManager instance containing the node to act on.
+        :param resource: A redfish resource (System or Manager) containing
+            virtual media.
+        :returns: An acceptable virtual media ID, or None if any ID
+            is acceptable.
+        """
+        return None
+
     def validate(self, task):
         """Validate the deployment information for the task's node.
 
@@ -509,14 +735,9 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :param task: A TaskManager instance with the node being checked
         :raises: MissingParameterValue if node is missing one or more required
             parameters
-        :raises: UnsupportedDriverExtension
         """
-        try:
-            self._validate_driver_info(task)
-        except exception.MissingParameterValue:
-            # Fall back to non-managed in-band inspection
-            raise exception.UnsupportedDriverExtension(
-                driver=task.node.driver, extension='inspection')
+        # No unmanaged fallback for virtual media
+        self._validate_driver_info(task)
 
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of deploy or rescue ramdisk over virtual media.
@@ -540,13 +761,14 @@ class RedfishVirtualMediaBoot(base.BootInterface):
             return
 
         d_info = _parse_driver_info(node)
-        managers = redfish_utils.get_system(task.node).managers
+        system = redfish_utils.get_system(task.node)
+        managers = system.managers
 
         self._validate_vendor(task, managers)
 
         if manager_utils.is_fast_track(task):
             if _has_vmedia_device(managers, sushy.VIRTUAL_MEDIA_CD,
-                                  inserted=True):
+                                  inserted=True, system=system):
                 LOG.debug('Fast track operation for node %s, not inserting '
                           'any devices', node.uuid)
                 return
@@ -584,7 +806,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
             removable = _has_vmedia_device(
                 managers,
                 # Prefer USB devices since floppies are outdated
-                [sushy.VIRTUAL_MEDIA_USBSTICK, sushy.VIRTUAL_MEDIA_FLOPPY])
+                [sushy.VIRTUAL_MEDIA_USBSTICK, sushy.VIRTUAL_MEDIA_FLOPPY],
+                system=system)
             if removable:
                 floppy_ref = image_utils.prepare_floppy_image(
                     task, params=ramdisk_params)
@@ -709,7 +932,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                                  'device': boot_devices.CDROM})
 
     def _attach_configdrive(self, task, managers):
-        configdrive = manager_utils.get_configdrive_image(task.node)
+        configdrive = configdrive_utils.get_configdrive_image(task.node)
         if not configdrive:
             return
 
@@ -880,14 +1103,8 @@ class RedfishHttpsBoot(base.BootInterface):
         :param task: A TaskManager instance with the node being checked
         :raises: MissingParameterValue if node is missing one or more required
             parameters
-        :raises: UnsupportedDriverExtension
         """
-        try:
-            self._validate_driver_info(task)
-        except exception.MissingParameterValue:
-            # Fall back to non-managed in-band inspection
-            raise exception.UnsupportedDriverExtension(
-                driver=task.node.driver, extension='inspection')
+        self._validate_driver_info(task)
 
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of the agent ramdisk.
