@@ -14,9 +14,11 @@
 #    under the License.
 
 import collections
+from datetime import timezone
 import time
 from urllib.parse import urlparse
 
+from dateutil import parser
 from oslo_log import log
 from oslo_utils import timeutils
 import sushy
@@ -89,6 +91,13 @@ BOOT_DEVICE_PERSISTENT_MAP = {
 BOOT_DEVICE_PERSISTENT_MAP_REV = {v: k for k, v in
                                   BOOT_DEVICE_PERSISTENT_MAP.items()}
 
+VENDORS_REQUIRING_FULL_BOOT_REQUEST = [
+    "american megatrends international",
+    "ami",
+    "asrockrack"
+    "redfish_compatible"
+]
+
 INDICATOR_MAP = {
     sushy.INDICATOR_LED_LIT: indicator_states.ON,
     sushy.INDICATOR_LED_OFF: indicator_states.OFF,
@@ -121,6 +130,10 @@ def _set_boot_device(task, system, device, persistent=False,
     :param http_boot_url: A string value to be sent to the sushy library,
                           which is sent to the BMC as the url to boot from.
     :raises: SushyError on an error from the Sushy library
+    Vendor-specific logic:
+    - Vendors listed in VENDORS_REQUIRING_FULL_BOOT_REQUEST:
+      Require setting full boot parameters
+      (mode, enabled, target) even if unchanged.
     """
 
     # The BMC handling of the persistent setting is vendor specific.
@@ -130,6 +143,13 @@ def _set_boot_device(task, system, device, persistent=False,
     # persistent setting must be set when setting the boot device
     # (see https://storyboard.openstack.org/#!/story/2008547).
     vendor = task.node.properties.get('vendor', None)
+    LOG.debug("Vendor : %(vendor)s node %(uuid)s",
+              {'vendor': vendor, 'uuid': task.node.uuid})
+    requires_full_boot_request = (
+        vendor and any(vendor_id in vendor.lower()
+                       for vendor_id in
+                       VENDORS_REQUIRING_FULL_BOOT_REQUEST)
+    )
     if vendor and vendor.lower() == 'supermicro':
         enabled = BOOT_DEVICE_PERSISTENT_MAP_REV[persistent]
         LOG.debug('Setting BootSourceOverrideEnable to %(enable)s '
@@ -145,8 +165,35 @@ def _set_boot_device(task, system, device, persistent=False,
     try:
         # NOTE(TheJulia): In sushy, it is uri, due to the convention used
         # in the standard. URL is used internally in ironic.
-        system.set_system_boot_options(device, enabled=enabled,
-                                       http_boot_uri=http_boot_url)
+        if requires_full_boot_request:
+            # Some vendors require sending all boot parameters every time
+            desired_mode = system.boot.get('mode') \
+                or sushy.BOOT_SOURCE_MODE_UEFI
+            desired_enabled = BOOT_DEVICE_PERSISTENT_MAP_REV[persistent]
+            current_enabled = system.boot.get('enabled') \
+                or sushy.BOOT_SOURCE_ENABLED_ONCE
+            current_target = system.boot.get('target') \
+                or sushy.BOOT_SOURCE_TARGET_NONE
+
+            LOG.debug('Vendor "%(vendor)s" requires full boot settings. '
+                      'Sending: mode=%(mode)s, enabled=%(enabled)s, '
+                      'target=%(target)s for node %(node)s',
+                      {'vendor': vendor, 'mode': desired_mode,
+                       'enabled': current_enabled, 'target': current_target,
+                       'node': task.node.uuid})
+
+            system.set_system_boot_options(
+                device,
+                mode=desired_mode,
+                enabled=enabled,
+                http_boot_uri=http_boot_url
+            )
+        else:
+            LOG.debug('Sending minimal Redfish boot device'
+                      ' change for node %(node)s',
+                      {'node': task.node.uuid})
+            system.set_system_boot_options(device, enabled=enabled,
+                                           http_boot_uri=http_boot_url)
     except sushy.exceptions.SushyError as e:
         if enabled == sushy.BOOT_SOURCE_ENABLED_CONTINUOUS:
             # NOTE(dtantsur): continuous boot device settings have been
@@ -334,13 +381,54 @@ class RedfishManagement(base.ManagementInterface):
         """
         system = redfish_utils.get_system(task.node)
 
+        vendor = task.node.properties.get('vendor', None)
+        requires_full_boot_request = (
+            vendor and any(vendor_id in vendor.lower()
+                           for vendor_id in
+                           VENDORS_REQUIRING_FULL_BOOT_REQUEST)
+        )
+
+        LOG.debug('Requested %(vendor)s to set boot mode'
+                  ' to "%(mode)s" for node %(node)s',
+                  {'mode': mode, 'node': task.node.uuid, 'vendor': vendor})
+
         # NOTE(dtantsur): check the readability of the current mode before
         # modifying anything. I suspect it can become None transiently after
         # the update, while we need to know if it is supported *at all*.
         get_mode_unsupported = (system.boot.get('mode') is None)
 
+        desired_mode = BOOT_MODE_MAP_REV[mode]
+        LOG.debug('Current boot mode read from Redfish for '
+                  'node %(node)s is: %(mode)s',
+                  {'node': task.node.uuid,
+                   'mode': desired_mode})
+
         try:
-            system.set_system_boot_options(mode=BOOT_MODE_MAP_REV[mode])
+
+            if requires_full_boot_request:
+                current_enabled = system.boot.get('enable') \
+                    or sushy.BOOT_SOURCE_ENABLED_ONCE
+                current_target = system.boot.get('enable') \
+                    or sushy.BOOT_SOURCE_TARGET_PXE
+
+                LOG.debug('Vendor "%(vendor)s" requires full boot settings. '
+                          'Sending: mode=%(mode)s, enabled=%(enabled)s, '
+                          'target=%(target)s for node %(node)s',
+                          {'vendor': vendor, 'mode': desired_mode,
+                           'enabled': current_enabled, 'node': task.node.uuid,
+                           'target': current_target
+                           })
+
+                system.set_system_boot_options(
+                    mode=desired_mode,
+                    enabled=current_enabled,
+                    target=current_target
+                )
+            else:
+                LOG.debug('Sending minimal Redfish boot mode '
+                          'change for node %(node)s',
+                          {'node': task.node.uuid})
+                system.set_system_boot_options(mode=desired_mode)
         except sushy.exceptions.SushyError as e:
             error_msg = (_('Setting boot mode to %(mode)s '
                            'failed for node %(node)s. '
@@ -441,9 +529,107 @@ class RedfishManagement(base.ManagementInterface):
 
     @staticmethod
     def _sensor2dict(resource, *fields):
-        return {field: getattr(resource, field)
-                for field in fields
-                if hasattr(resource, field)}
+        return {
+            field: attr.value if hasattr(attr, 'value') else attr
+            for field in fields
+            if hasattr(resource, field)
+            for attr in [getattr(resource, field)]
+        }
+
+    @base.clean_step(priority=0, abortable=False, argsinfo={
+        'target_datetime': {
+            'description': 'The datetime to set in ISO8601 format',
+            'required': True
+        },
+        'datetime_local_offset': {
+            'description': 'The local time offset from UTC',
+            'required': False
+        }
+    })
+    @task_manager.require_exclusive_lock
+    def set_bmc_clock(self, task, target_datetime, datetime_local_offset=None):
+        """Set the BMC clock using Redfish Manager resource.
+
+        :param task: a TaskManager instance containing the node to act on.
+        :param target_datetime: The datetime to set in ISO8601 format
+        :param datetime_local_offset: The local time offset from UTC (optional)
+        :raises: RedfishError if the operation fails
+        """
+        try:
+            system = redfish_utils.get_system(task.node)
+            manager = redfish_utils.get_manager(task.node, system)
+            LOG.debug("Setting BMC clock to %s (offset: %s)",
+                      target_datetime, datetime_local_offset)
+            manager._conn.timeout = 30
+            manager.set_datetime(
+                target_datetime,
+                datetime_local_offset
+            )
+
+            manager.refresh()
+            if manager.datetime != target_datetime:
+                raise exception.RedfishError(
+                    "BMC clock update failed: mismatch after setting datetime")
+
+            LOG.info(
+                "Successfully updated BMC clock for node %s",
+                task.node.uuid
+            )
+        except Exception as e:
+            LOG.exception("BMC clock update failed: %s", e)
+            raise exception.RedfishError(error=str(e))
+
+    @base.verify_step(priority=1)
+    @task_manager.require_exclusive_lock
+    def verify_bmc_clock(self, task):
+        """Verify and auto-set the BMC clock to the current UTC time.
+
+        This step compares the system UTC time to the BMC's Redfish datetime.
+        If the difference exceeds 1 second, it attempts to sync the time.
+        Verification fails only if the BMC time remains incorrect
+        after the update.
+        """
+        if not CONF.redfish.enable_verify_bmc_clock:
+            LOG.info("Skipping BMC clock verify step: disabled via config")
+            return
+
+        try:
+            system_time = timeutils.utcnow().replace(
+                tzinfo=timezone.utc).isoformat()
+            system = redfish_utils.get_system(task.node)
+            manager = redfish_utils.get_manager(task.node, system)
+            manager.refresh()
+
+            manager_time = parser.isoparse(manager.datetime)
+            local_time = parser.isoparse(system_time)
+
+            LOG.debug("BMC time: %s, Local time: %s",
+                      manager_time, local_time)
+            LOG.debug("manager.datetime_local_offset: %s",
+                      manager.datetimelocaloffset)
+
+            # Fail if the BMC clock differs from system time
+            # by more than 1 second
+            if abs((manager_time - local_time).total_seconds()) > 1:
+                LOG.info("BMC clock is out of sync. Updating...")
+                manager.set_datetime(system_time,
+                                     datetime_local_offset="+00:00")
+                manager.refresh()
+
+                updated_time = parser.isoparse(manager.datetime)
+                if abs((updated_time - local_time).total_seconds()) > 1:
+                    raise exception.RedfishError(
+                        "BMC clock still incorrect after update")
+
+            LOG.info("BMC clock update successful for node %s",
+                     task.node.uuid)
+
+        except Exception as e:
+            LOG.exception("BMC clock auto-update failed during verify: %s", e)
+            raise exception.NodeVerifyFailure(
+                node=getattr(task.node, 'uuid', 'unknown'),
+                reason="BMC clock verify step failed: %s" % str(e)
+            )
 
     @classmethod
     def _get_sensors_fan(cls, chassis):
@@ -548,6 +734,14 @@ class RedfishManagement(base.ManagementInterface):
 
         system = redfish_utils.get_system(node)
 
+        baremetal_fields = {
+            'Manufacturer': system.manufacturer,
+            'Model': system.model,
+            'UUID': system.uuid
+        }
+
+        sensors['Extra'] = baremetal_fields
+
         for chassis in system.chassis:
             try:
                 sensors['Fan'].update(self._get_sensors_fan(chassis))
@@ -581,7 +775,6 @@ class RedfishManagement(base.ManagementInterface):
             LOG.debug("Failed reading drive information for node "
                       "%(node)s: %(error)s", {'node': node.uuid,
                                               'error': exc})
-
         LOG.debug("Gathered sensor data: %(sensors)s", {'sensors': sensors})
 
         return sensors
@@ -1165,11 +1358,20 @@ class RedfishManagement(base.ManagementInterface):
             runtime error.
         :raises: UnsupportedDriverExtension if secure boot is
                  not supported by the hardware.
-        :returns: Boolean
+        :returns: Boolean or None if status cannot be retrieved
         """
         system = redfish_utils.get_system(task.node)
         try:
             return system.secure_boot.enabled
+        except sushy.exceptions.AccessError as e:
+            if 'OemLicenseNotPassed' in str(e):
+                # NOTE(cid): Supermicro gear requires a license to report
+                # secure boot status.
+                LOG.info("Secure boot status request through Redfish failed "
+                         "for node %(node)s: %(error)s",
+                         {'node': task.node.uuid, 'error': e})
+                return None
+            raise
         except sushy.exceptions.MissingAttributeError:
             raise exception.UnsupportedDriverExtension(
                 driver=task.node.driver, extension='get_secure_boot_state')
