@@ -83,7 +83,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.39: Add firmware_interface field
     # Version 1.40: Add service_step field
     # Version 1.41: Add disable_power_off field
-    VERSION = '1.41'
+    # Version 1.42: Moves multiple methods to be remotable methods.
+    # Version 1.43: Add instance_name field
+    VERSION = '1.43'
 
     dbapi = db_api.get_instance()
 
@@ -184,6 +186,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'shard': object_fields.StringField(nullable=True),
         'parent_node': object_fields.StringField(nullable=True),
         'disable_power_off': objects.fields.BooleanField(nullable=True),
+        'instance_name': object_fields.StringField(nullable=True),
     }
 
     def as_dict(self, secure=False, mask_configdrive=True):
@@ -240,11 +243,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                 fields=['trait', 'version'])
             self.traits.obj_reset_changes()
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get(cls, context, node_id):
         """Find a node based on its id or uuid and return a Node object.
 
@@ -259,11 +258,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         else:
             raise exception.InvalidIdentity(identity=node_id)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_id(cls, context, node_id):
         """Find a node based on its integer ID and return a Node object.
 
@@ -276,11 +271,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_uuid(cls, context, uuid):
         """Find a node based on UUID and return a Node object.
 
@@ -293,11 +284,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_name(cls, context, name):
         """Find a node based on name and return a Node object.
 
@@ -310,11 +297,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_instance_uuid(cls, context, instance_uuid):
         """Find a node based on the instance UUID and return a Node object.
 
@@ -327,11 +310,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def list(cls, context, limit=None, marker=None, sort_key=None,
              sort_dir=None, filters=None, fields=None):
         """Return a list of Node objects.
@@ -367,10 +346,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                                            fields=target_fields)
         return cls._from_db_object_list(context, db_nodes, target_fields)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
+    # NOTE(TheJulia): The choice to not make this a remotable method is
+    # explicit in that locks are intended only for a conductor. If we choose
+    # to change this, we need reconsider the locking model.
     @classmethod
     def reserve(cls, context, tag, node_id):
         """Get and reserve a node.
@@ -390,10 +368,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
+    # NOTE(TheJulia): The choice to not make this a remotable method is
+    # explicit in that locks are intended only for a conductor. If we choose
+    # to change this, we need reconsider the locking model.
     @classmethod
     def release(cls, context, tag, node_id):
         """Release the reservation on a node.
@@ -406,10 +383,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         """
         cls.dbapi.release_node(tag, node_id)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def create(self, context=None):
         """Create a Node record in the DB.
 
@@ -433,10 +407,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         db_node = self.dbapi.create_node(values)
         self._from_db_object(self._context, self, db_node)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def destroy(self, context=None):
         """Delete the Node from the DB.
 
@@ -450,10 +421,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.dbapi.destroy_node(self.uuid)
         self.obj_reset_changes()
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def save(self, context=None):
         """Save updates to this Node.
 
@@ -519,10 +487,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             utils.validate_conductor_group(fields['conductor_group'])
             fields['conductor_group'] = fields['conductor_group'].lower()
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def refresh(self, context=None):
         """Refresh the object by re-fetching from the DB.
 
@@ -537,15 +502,11 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.obj_refresh(current)
         self.obj_reset_changes()
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
     def touch_provisioning(self, context=None):
         """Touch the database record to mark the provisioning as alive."""
         self.dbapi.touch_node_provisioning(self.id)
 
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_port_addresses(cls, context, addresses):
         """Get a node by associated port addresses.
 
@@ -675,6 +636,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         Version 1.39: firmware_interface field was added. Its default value is
             None. For versions prior to this, it should be set to None (or
             removed).
+        Version 1.43: instance_name field was added. Its default value is
+            None. For versions prior to this, it should be set to None (or
+            removed).
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -691,7 +655,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                   ('owner', 30), ('allocation_id', 31), ('description', 32),
                   ('retired_reason', 33), ('lessee', 34), ('boot_mode', 36),
                   ('secure_boot', 36), ('shard', 37),
-                  ('firmware_interface', 39)]
+                  ('firmware_interface', 39), ('instance_name', 43)]
 
         for name, minor in fields:
             self._adjust_field_to_version(name, None, target_version,
@@ -773,6 +737,17 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.properties[key] = value
         self._changed_fields.add('properties')
 
+    def list_child_node_ids(self, exclude_dedicated_power=False):
+        """Returns a list of node IDs for child nodes, if any.
+
+        :param exclude_dedicated_power: Boolean, Default False, if the list
+            should exclude nodes which are independently powered.
+        :returns: A list of any node_id values discovered in the database.
+        """
+        return self.dbapi.get_child_node_ids_by_parent_uuid(
+            self.uuid,
+            exclude_dedicated_power=exclude_dedicated_power)
+
 
 @base.IronicObjectRegistry.register
 class NodePayload(notification.NotificationPayloadBase):
