@@ -25,6 +25,7 @@ from ironic.db import api as dbapi
 from ironic.objects import base
 from ironic.objects import fields as object_fields
 from ironic.objects import notification
+from ironic.objects import Port
 
 
 @base.IronicObjectRegistry.register
@@ -37,7 +38,10 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
     #              internal_info['tenant_vif_port_id'] (not an explicit db
     #              change)
     # Version 1.5: Add node_uuid field
-    VERSION = '1.5'
+    # Version 1.6: Relevant methods changed to be remotable methods.
+    # Version 1.7: Add physical_network field
+    # Version 1.8: Add category field
+    VERSION = '1.8'
 
     dbapi = dbapi.get_instance()
 
@@ -53,8 +57,39 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         'standalone_ports_supported': object_fields.BooleanField(),
         'mode': object_fields.StringField(nullable=True),
         'properties': object_fields.FlexibleDictField(nullable=True),
+        'physical_network': object_fields.StringField(nullable=True),
+        'category': object_fields.StringField(nullable=True),
     }
 
+    # TODO(clif): Abstract this, already exists in Port object.
+    def _convert_field_by_version(self, field_name, introduced_version,
+                                  target_version,
+                                  remove_unavailable_fields=True,
+                                  default_value=None):
+        """Convert a field based on version compatibility.
+
+        :param field_name: Name of the field to convert
+        :param introduced_version: Version tuple when the field was introduced
+        :param target_version: Target version to convert to
+        :param remove_unavailable_fields: Whether to remove fields not in
+            target version
+        :param default_value: Default value to set if field is not set
+        """
+        field_is_set = self.obj_attr_is_set(field_name)
+        if target_version >= introduced_version:
+            # Target version supports this field. Set it to its default
+            # value if it is not set.
+            if not field_is_set:
+                setattr(self, field_name, default_value)
+        elif field_is_set:
+            # Target version does not support this field, and it is set.
+            if remove_unavailable_fields:
+                # (De)serialising: remove unavailable fields.
+                delattr(self, field_name)
+            elif getattr(self, field_name) is not default_value:
+                # DB: set unavailable fields to their default.
+                setattr(self, field_name, default_value)
+
     def _convert_to_version(self, target_version,
                             remove_unavailable_fields=True):
         """Convert to the target version.
@@ -68,6 +103,15 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
             .extra value to internal_info. There is nothing to do here when
             downgrading to an older version.
 
+        Version 1.5: remove node_uuid for unsupported versions if
+            remove_unavailable_fields is True.
+
+        Version 1.7: remove physical_network for unsupported versions if
+            remove_unavailable_fields is True.
+
+        Version 1.8: remove category for unsupported versions if
+            remove_unavailable_fields is True.
+
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
             unavailable in the target version; set this to True when
@@ -86,11 +130,20 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
                         internal_info['tenant_vif_port_id'] = vif
                         self.internal_info = internal_info
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+        # Convert the node_uuid field.
+        self._convert_field_by_version('node_uuid', (1, 5), target_version,
+                                       remove_unavailable_fields)
+
+        # Convert the physical_network field.
+        self._convert_field_by_version('physical_network', (1, 7),
+                                       target_version,
+                                       remove_unavailable_fields)
+
+        # Convert the category field.
+        self._convert_field_by_version('category', (1, 8), target_version,
+                                       remove_unavailable_fields)
+
+    @object_base.remotable_classmethod
     def get(cls, context, portgroup_ident):
         """Find a portgroup based on its id, uuid, name or address.
 
@@ -111,11 +164,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         else:
             raise exception.InvalidIdentity(identity=portgroup_ident)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_id(cls, context, portgroup_id):
         """Find a portgroup by its integer ID and return a Portgroup object.
 
@@ -130,11 +179,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         portgroup = cls._from_db_object(context, cls(), db_portgroup)
         return portgroup
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_uuid(cls, context, uuid):
         """Find a portgroup by UUID and return a :class:`Portgroup` object.
 
@@ -149,11 +194,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         portgroup = cls._from_db_object(context, cls(), db_portgroup)
         return portgroup
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_address(cls, context, address, project=None):
         """Find portgroup by address and return a :class:`Portgroup` object.
 
@@ -170,11 +211,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         portgroup = cls._from_db_object(context, cls(), db_portgroup)
         return portgroup
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def get_by_name(cls, context, name):
         """Find portgroup based on name and return a :class:`Portgroup` object.
 
@@ -189,11 +226,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         portgroup = cls._from_db_object(context, cls(), db_portgroup)
         return portgroup
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def list(cls, context, limit=None, marker=None,
              sort_key=None, sort_dir=None, project=None,
              conductor_groups=None, filters=None):
@@ -223,11 +256,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
             conductor_groups=conductor_groups)
         return cls._from_db_object_list(context, db_portgroups)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable_classmethod
-    @classmethod
+    @object_base.remotable_classmethod
     def list_by_node_id(cls, context, node_id, limit=None, marker=None,
                         sort_key=None, sort_dir=None, project=None):
         """Return a list of Portgroup objects associated with a given node ID.
@@ -252,10 +281,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
                                                             project=project)
         return cls._from_db_object_list(context, db_portgroups)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def create(self, context=None):
         """Create a Portgroup record in the DB.
 
@@ -276,10 +302,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         db_portgroup = self.dbapi.get_portgroup_by_id(db_portgroup['id'])
         self._from_db_object(self._context, self, db_portgroup)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def destroy(self, context=None):
         """Delete the Portgroup from the DB.
 
@@ -295,10 +318,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.dbapi.destroy_portgroup(self.id)
         self.obj_reset_changes()
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def save(self, context=None):
         """Save updates to this Portgroup.
 
@@ -318,10 +338,7 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         updated_portgroup = self.dbapi.update_portgroup(self.uuid, updates)
         self._from_db_object(self._context, self, updated_portgroup)
 
-    # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
-    # methods can be used in the future to replace current explicit RPC calls.
-    # Implications of calling new remote procedures should be thought through.
-    # @object_base.remotable
+    @object_base.remotable
     def refresh(self, context=None):
         """Loads updates for this Portgroup.
 
@@ -342,6 +359,27 @@ class Portgroup(base.IronicObject, object_base.VersionedObjectDictCompat):
         self.obj_refresh(current)
         self.obj_reset_changes()
 
+    @object_base.remotable
+    def update_physical_network(self, new_physical_network, context=None):
+        """Updates the phyiscal_network for this Portgroup and its Ports.
+
+        :param context: Security context. NOTE: This should only
+                        be used internally by the indirection_api.
+                        Unfortunately, RPC requires context as the first
+                        argument, even though we don't use it.
+                        A context should be set when instantiating the
+                        object, e.g.: Portgroup(context)
+        :raises: PortgroupNotFound
+
+        """
+        self.physical_network = new_physical_network
+        self.save(context=context)
+
+        for p in Port.list_by_portgroup_id(context, self.id):
+            if new_physical_network != p.physical_network:
+                p.physical_network = new_physical_network
+                p.save(context=context)
+
 
 @base.IronicObjectRegistry.register
 class PortgroupCRUDNotification(notification.NotificationBase):
@@ -357,14 +395,18 @@ class PortgroupCRUDNotification(notification.NotificationBase):
 @base.IronicObjectRegistry.register
 class PortgroupCRUDPayload(notification.NotificationPayloadBase):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Add physical_network field
+    # Version 1.2: Add category field
+    VERSION = '1.2'
 
     SCHEMA = {
         'address': ('portgroup', 'address'),
+        'category': ('portgroup', 'category'),
         'extra': ('portgroup', 'extra'),
         'mode': ('portgroup', 'mode'),
         'name': ('portgroup', 'name'),
         'properties': ('portgroup', 'properties'),
+        'physical_network': ('portgroup', 'physical_network'),
         'standalone_ports_supported': ('portgroup',
                                        'standalone_ports_supported'),
         'created_at': ('portgroup', 'created_at'),
@@ -374,10 +416,12 @@ class PortgroupCRUDPayload(notification.NotificationPayloadBase):
 
     fields = {
         'address': object_fields.MACAddressField(nullable=True),
+        'category': object_fields.StringField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
         'mode': object_fields.StringField(nullable=True),
         'name': object_fields.StringField(nullable=True),
         'node_uuid': object_fields.UUIDField(),
+        'physical_network': object_fields.StringField(nullable=True),
         'properties': object_fields.FlexibleDictField(nullable=True),
         'standalone_ports_supported': object_fields.BooleanField(
             nullable=True),
