@@ -64,8 +64,21 @@ class TestLookup(test_api_base.BaseApiTest):
         node.driver_internal_info = driver_internal
         self.mock_get_node_with_token.return_value = node
 
-    def _check_config(self, data):
+    def _check_config(self, data, skip_bmc_detect=False):
         expected_config = {
+            'agent_containers': {
+                'allow_arbitrary_containers': CONF.agent_containers
+                .allow_arbitrary_containers,
+                'allowed_containers': CONF.agent_containers
+                .allowed_containers,
+                'container_steps_file': CONF.agent_containers
+                .container_steps_file,
+                'runner': CONF.agent_containers.runner,
+                'pull_options': CONF.agent_containers.pull_options,
+                'run_options': CONF.agent_containers.run_options,
+                'container_conf_file': CONF.agent_containers
+                .container_conf_file,
+            },
             'metrics': {
                 'backend': 'statsd',
                 'prepend_host': CONF.metrics.agent_prepend_host,
@@ -84,6 +97,7 @@ class TestLookup(test_api_base.BaseApiTest):
             'agent_md5_checksum_enable': CONF.agent.allow_md5_checksum,
             'disable_deep_image_inspection': CONF.conductor.disable_deep_image_inspection,  # noqa
             'permitted_image_formats': CONF.conductor.permitted_image_formats,
+            'agent_skip_bmc_detect': skip_bmc_detect,
         }
         self.assertEqual(expected_config, data['config'])
         self.assertIsNotNone(data['config']['agent_token'])
@@ -96,6 +110,20 @@ class TestLookup(test_api_base.BaseApiTest):
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
 
+    def test_invalid_input_addresses(self):
+        response = self.get_json(
+            '/lookup?addresses=meow1',
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
+    def test_invalid_input_node_uuid(self):
+        response = self.get_json(
+            '/lookup?node_uuid=meow1',
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
     def test_not_found(self):
         response = self.get_json(
             '/lookup?addresses=%s' % ','.join(self.addresses),
@@ -199,6 +227,85 @@ class TestLookup(test_api_base.BaseApiTest):
                 headers={api_base.Version.string: str(api_v1.max_version())})
             self.assertEqual(self.node.uuid, data['node']['uuid'])
 
+    def test_bmc_detect_skip_for_redfish(self):
+        """Test BMC detection skip is enabled for Redfish interface."""
+        self.node.management_interface = 'redfish'
+        self.node.save()
+        self._set_secret_mock(self.node, 'test-token')
+
+        data = self.get_json(
+            '/lookup?node_uuid=%s' % self.node.uuid,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(self.node.uuid, data['node']['uuid'])
+        self._check_config(data, skip_bmc_detect=True)
+
+    def test_bmc_detect_skip_for_ilo_variants(self):
+        """Test BMC detection skip is enabled for iLO variants."""
+        for ilo_interface in ['ilo', 'ilo5', 'ilo6']:
+            self.node.management_interface = ilo_interface
+            self.node.save()
+            self._set_secret_mock(self.node, 'test-token')
+
+            data = self.get_json(
+                '/lookup?node_uuid=%s' % self.node.uuid,
+                headers={api_base.Version.string: str(api_v1.max_version())})
+
+            self.assertEqual(self.node.uuid, data['node']['uuid'])
+            self._check_config(data, skip_bmc_detect=True)
+
+    def test_bmc_detect_skip_for_idrac_redfish(self):
+        """Test BMC detection skip is enabled for iDRAC Redfish."""
+        self.node.management_interface = 'idrac-redfish'
+        self.node.save()
+        self._set_secret_mock(self.node, 'test-token')
+
+        data = self.get_json(
+            '/lookup?node_uuid=%s' % self.node.uuid,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(self.node.uuid, data['node']['uuid'])
+        self._check_config(data, skip_bmc_detect=True)
+
+    def test_bmc_detect_skip_for_irmc(self):
+        """Test BMC detection skip is enabled for iRMC."""
+        self.node.management_interface = 'irmc'
+        self.node.save()
+        self._set_secret_mock(self.node, 'test-token')
+
+        data = self.get_json(
+            '/lookup?node_uuid=%s' % self.node.uuid,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(self.node.uuid, data['node']['uuid'])
+        self._check_config(data, skip_bmc_detect=True)
+
+    def test_bmc_detect_not_skipped_for_ipmi(self):
+        """Test BMC detection is NOT skipped for IPMI interface."""
+        self.node.management_interface = 'ipmitool'
+        self.node.save()
+        self._set_secret_mock(self.node, 'test-token')
+
+        data = self.get_json(
+            '/lookup?node_uuid=%s' % self.node.uuid,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(self.node.uuid, data['node']['uuid'])
+        self._check_config(data, skip_bmc_detect=False)
+
+    def test_bmc_detect_not_skipped_for_fake(self):
+        """Test BMC detection is NOT skipped for fake interface."""
+        self.node.management_interface = 'fake'
+        self.node.save()
+        self._set_secret_mock(self.node, 'test-token')
+
+        data = self.get_json(
+            '/lookup?node_uuid=%s' % self.node.uuid,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+
+        self.assertEqual(self.node.uuid, data['node']['uuid'])
+        self._check_config(data, skip_bmc_detect=False)
+
 
 @mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for',
                    lambda *n: 'test-topic')
@@ -234,6 +341,21 @@ class TestHeartbeat(test_api_base.BaseApiTest):
                                                'x', None, None, None,
                                                topic='test-topic')
 
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_ok_for_anaconda(self, mock_heartbeat):
+        node = obj_utils.create_test_node(self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': '',
+             'agent_token': 'x'},
+            headers={api_base.Version.string: str(api_v1.max_version())})
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual(b'', response.body)
+        mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
+                                               node.uuid, '', None,
+                                               'x', None, None, None,
+                                               topic='test-topic')
+
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_ok_with_json(self, mock_heartbeat):
         headers = {api_base.Version.string: '1.90'}
@@ -413,6 +535,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
 class TestLookupScopedRBAC(TestLookup):
 
     """Test class to execute the Lookup tests with RBAC enforcement."""
+
     def setUp(self):
         super(TestLookupScopedRBAC, self).setUp()
 
@@ -427,6 +550,7 @@ class TestLookupScopedRBAC(TestLookup):
 class TestHeartbeatScopedRBAC(TestHeartbeat):
 
     """Test class to execute the Heartbeat tests with RBAC enforcement."""
+
     def setUp(self):
         super(TestHeartbeatScopedRBAC, self).setUp()
 
