@@ -24,7 +24,7 @@ from unittest import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import fileutils
-from oslo_utils.imageutils import format_inspector as image_format_inspector
+from oslo_utils.imageutils import format_inspector
 
 from ironic.common import exception
 from ironic.common.glance_service import service_utils as glance_utils
@@ -42,9 +42,11 @@ class IronicImagesTestCase(base.TestCase):
     class FakeImgInfo(object):
         pass
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     @mock.patch.object(builtins, 'open', autospec=True)
-    def test_fetch_image_service(self, open_mock, image_service_mock):
+    def test_fetch_image_service(self, open_mock, image_service_mock,
+                                 mock_zstd):
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -56,12 +58,16 @@ class IronicImagesTestCase(base.TestCase):
                                                    context='context')
         image_service_mock.return_value.download.assert_called_once_with(
             'image_href', 'file')
+        mock_zstd.assert_called_once_with('path')
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw(self, open_mock, image_to_raw_mock,
-                                           image_service_mock):
+                                           image_service_mock,
+                                           mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -73,7 +79,9 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(fileutils, 'compute_file_checksum',
                        autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
@@ -81,7 +89,8 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw_with_checksum(
             self, open_mock, image_to_raw_mock,
-            image_service_mock, mock_checksum):
+            image_service_mock, mock_checksum, mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -96,7 +105,9 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(fileutils, 'compute_file_checksum',
                        autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
@@ -104,7 +115,9 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_with_checksum_mismatch(
             self, open_mock, image_to_raw_mock,
-            image_service_mock, mock_checksum):
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -121,7 +134,9 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         # If the checksum fails, then we don't attempt to convert the image.
         image_to_raw_mock.assert_not_called()
+        mock_zstd.assert_not_called()
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(fileutils, 'compute_file_checksum',
                        autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
@@ -129,7 +144,9 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw_no_checksum_algo(
             self, open_mock, image_to_raw_mock,
-            image_service_mock, mock_checksum):
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -144,7 +161,9 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
 
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
     @mock.patch.object(fileutils, 'compute_file_checksum',
                        autospec=True)
     @mock.patch.object(image_service, 'get_image_service', autospec=True)
@@ -152,7 +171,9 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(builtins, 'open', autospec=True)
     def test_fetch_image_service_force_raw_combined_algo(
             self, open_mock, image_to_raw_mock,
-            image_service_mock, mock_checksum):
+            image_service_mock, mock_checksum,
+            mock_zstd):
+        image_service_mock.return_value.transfer_verified_checksum = None
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'file'
         open_mock.return_value = mock_file_handle
@@ -167,9 +188,41 @@ class IronicImagesTestCase(base.TestCase):
             'image_href', 'file')
         image_to_raw_mock.assert_called_once_with(
             'image_href', 'path', 'path.part')
+        mock_zstd.assert_called_once_with('path')
 
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
+    @mock.patch.object(images, '_handle_zstd_compression', autospec=True)
+    @mock.patch.object(fileutils, 'compute_file_checksum',
                        autospec=True)
+    @mock.patch.object(image_service, 'get_image_service', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_fetch_image_service_auth_data_checksum(
+            self, open_mock, image_to_raw_mock,
+            svc_mock, mock_checksum,
+            mock_zstd):
+        svc_mock.return_value.transfer_verified_checksum = 'f00'
+        svc_mock.return_value.is_auth_set_needed = True
+        mock_file_handle = mock.MagicMock(spec=io.BytesIO)
+        mock_file_handle.__enter__.return_value = 'file'
+        open_mock.return_value = mock_file_handle
+        mock_checksum.return_value = 'f00'
+
+        images.fetch('context', 'image_href', 'path', force_raw=True,
+                     checksum='sha512:f00', image_auth_data='meow')
+        # In this case, the image service does the checksum so we know
+        # we don't need to do a checksum pass as part of the common image
+        # handling code path.
+        mock_checksum.assert_not_called()
+        open_mock.assert_called_once_with('path', 'wb')
+        svc_mock.return_value.download.assert_called_once_with(
+            'image_href', 'file')
+        image_to_raw_mock.assert_called_once_with(
+            'image_href', 'path', 'path.part')
+        svc_mock.return_value.set_image_auth.assert_called_once_with(
+            'image_href', 'meow')
+        mock_zstd.assert_called_once_with('path')
+
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_not_permitted_format(self, detect_format_mock):
         info = mock.MagicMock()
         # In the case the image looks okay, but it is not in our permitted
@@ -183,13 +236,12 @@ class IronicImagesTestCase(base.TestCase):
         detect_format_mock.assert_called_once_with('path_tmp')
         self.assertIn("The requested image is not valid for use.", str(e))
 
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_fails_safety_check(self, detect_format_mock):
         info = mock.MagicMock()
         info.__str__.return_value = 'qcow2'
         info.safety_check.side_effect = \
-            image_format_inspector.SafetyCheckFailed({"I'm a teapot": True})
+            format_inspector.SafetyCheckFailed({"I'm a teapot": True})
         detect_format_mock.return_value = info
 
         e = self.assertRaises(exception.ImageUnacceptable, images.image_to_raw,
@@ -203,8 +255,7 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw(self, detect_format_mock, convert_image_mock,
                           unlink_mock, rename_mock):
         CONF.set_override('force_raw_images', True)
@@ -232,8 +283,7 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_gpt(self, detect_format_mock, convert_image_mock,
                           unlink_mock, rename_mock):
         CONF.set_override('force_raw_images', True)
@@ -261,8 +311,7 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_gpt_backward_compatibility(self, detect_format_mock,
                                                  convert_image_mock,
                                                  unlink_mock, rename_mock):
@@ -293,8 +342,7 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_safety_check_disabled(
             self, detect_format_mock, convert_image_mock,
             unlink_mock, rename_mock):
@@ -313,7 +361,7 @@ class IronicImagesTestCase(base.TestCase):
         images.image_to_raw('image_href', 'path', 'path_tmp')
         info.safety_check.assert_not_called()
         detect_format_mock.assert_has_calls([
-            mock.call('path')])
+            mock.call('path_tmp')])
         self.assertEqual(2, info.__str__.call_count)
         convert_image_mock.assert_called_once_with('path_tmp',
                                                    'path.converted', 'raw',
@@ -324,8 +372,7 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(os, 'rename', autospec=True)
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_safety_check_disabled_fails_to_convert(
             self, detect_format_mock, convert_image_mock,
             unlink_mock, rename_mock):
@@ -343,7 +390,7 @@ class IronicImagesTestCase(base.TestCase):
         info.safety_check.assert_not_called()
         self.assertEqual(2, info.__str__.call_count)
         detect_format_mock.assert_has_calls([
-            mock.call('path')])
+            mock.call('path_tmp')])
         convert_image_mock.assert_called_once_with('path_tmp',
                                                    'path.converted', 'raw',
                                                    source_format='vmdk')
@@ -352,8 +399,7 @@ class IronicImagesTestCase(base.TestCase):
 
     @mock.patch.object(os, 'unlink', autospec=True)
     @mock.patch.object(qemu_img, 'convert_image', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_not_raw_after_conversion(self, detect_format_mock,
                                                    convert_image_mock,
                                                    unlink_mock):
@@ -375,8 +421,7 @@ class IronicImagesTestCase(base.TestCase):
             mock.call('path.converted')])
 
     @mock.patch.object(os, 'rename', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_already_raw_format(self, detect_format_mock,
                                              rename_mock):
         info = mock.MagicMock()
@@ -391,8 +436,7 @@ class IronicImagesTestCase(base.TestCase):
         detect_format_mock.assert_called_once_with('path_tmp')
 
     @mock.patch.object(os, 'rename', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_already_gpt_format(self, detect_format_mock,
                                              rename_mock):
         info = mock.MagicMock()
@@ -407,8 +451,7 @@ class IronicImagesTestCase(base.TestCase):
         detect_format_mock.assert_called_once_with('path_tmp')
 
     @mock.patch.object(os, 'rename', autospec=True)
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_image_to_raw_already_iso(self, detect_format_mock,
                                       rename_mock):
         info = mock.MagicMock()
@@ -438,13 +481,14 @@ class IronicImagesTestCase(base.TestCase):
     @mock.patch.object(images, 'image_show', autospec=True)
     def test_download_size(self, show_mock):
         show_mock.return_value = {'size': 123456}
-        size = images.download_size('context', 'image_href', 'image_service')
+        size = images.download_size('context', 'image_href', 'image_service',
+                                    image_auth_data='meow')
         self.assertEqual(123456, size)
         show_mock.assert_called_once_with('context', 'image_href',
-                                          'image_service')
+                                          image_service='image_service',
+                                          image_auth_data='meow')
 
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_converted_size_estimate_default(self, image_info_mock):
         info = self.FakeImgInfo()
         info.actual_size = 2
@@ -454,8 +498,7 @@ class IronicImagesTestCase(base.TestCase):
         image_info_mock.assert_called_once_with('path')
         self.assertEqual(4, size)
 
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_converted_size_estimate_custom(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
@@ -466,8 +509,7 @@ class IronicImagesTestCase(base.TestCase):
         image_info_mock.assert_called_once_with('path')
         self.assertEqual(6, size)
 
-    @mock.patch.object(image_format_inspector, 'detect_file_format',
-                       autospec=True)
+    @mock.patch.object(images, 'detect_file_format', autospec=True)
     def test_converted_size_estimate_raw_smaller(self, image_info_mock):
         CONF.set_override('raw_image_growth_factor', 3)
         info = self.FakeImgInfo()
@@ -540,6 +582,25 @@ class IronicImagesTestCase(base.TestCase):
         mock_igi.assert_called_once_with(image_source)
         mock_gip.assert_called_once_with('context', image_source)
 
+    @mock.patch.object(images, 'get_image_properties', autospec=True)
+    @mock.patch.object(image_service, 'is_container_registry_url',
+                       autospec=True)
+    @mock.patch.object(glance_utils, 'is_glance_image', autospec=True)
+    def test_is_whole_disk_image_whole_disk_image_oci(self, mock_igi,
+                                                      mock_ioi,
+                                                      mock_gip):
+        mock_igi.return_value = False
+        mock_ioi.return_value = True
+        mock_gip.return_value = {}
+        instance_info = {'image_source': 'oci://image'}
+        image_source = instance_info['image_source']
+        is_whole_disk_image = images.is_whole_disk_image('context',
+                                                         instance_info)
+        self.assertTrue(is_whole_disk_image)
+        mock_igi.assert_called_once_with(image_source)
+        mock_ioi.assert_called_once_with(image_source)
+        mock_gip.assert_not_called()
+
     @mock.patch.object(images, 'get_image_properties', autospec=True)
     @mock.patch.object(glance_utils, 'is_glance_image', autospec=True)
     def test_is_whole_disk_image_partition_non_glance(self, mock_igi,
@@ -563,6 +624,7 @@ class IronicImagesTestCase(base.TestCase):
                                                        mock_gip,
                                                        mock_validate):
         mock_igi.return_value = False
+        mock_validate.return_value.url = 'http://whole-disk-image'
         instance_info = {
             'image_source': 'http://whole-disk-image'}
         is_whole_disk_image = images.is_whole_disk_image('context',
@@ -604,22 +666,6 @@ class IronicImagesTestCase(base.TestCase):
         self.assertTrue(images.is_source_a_path('context', 'http://foo/bar'))
         validate_mock.assert_called_once_with(mock.ANY, 'http://foo/bar')
 
-    @mock.patch.object(images, 'LOG', autospec=True)
-    @mock.patch.object(image_service.HttpImageService, 'validate_href',
-                       autospec=True)
-    def test_is_source_a_path_redirect(self, validate_mock, log_mock):
-        url = 'http://foo/bar'
-        redirect_url = url + '/'
-        validate_mock.side_effect = exception.ImageRefIsARedirect(
-            url, redirect_url)
-        self.assertTrue(images.is_source_a_path('context', url))
-        log_mock.debug.assert_called_once_with('Received a URL redirect when '
-                                               'attempting to evaluate image '
-                                               'reference http://foo/bar '
-                                               'pointing to http://foo/bar/. '
-                                               'This may, or may not be '
-                                               'recoverable.')
-
     @mock.patch.object(image_service.HttpImageService, 'validate_href',
                        autospec=True)
     def test_is_source_a_path_other_error(self, validate_mock):
@@ -627,6 +673,85 @@ class IronicImagesTestCase(base.TestCase):
         validate_mock.side_effect = OSError
         self.assertIsNone(images.is_source_a_path('context', url))
 
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(shutil, 'move', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test__hanlde_zstd_compression(self, mock_open, mock_move,
+                                      mock_exec):
+        mock_file_handle = mock.Mock()
+        mock_file_handle.read.return_value = b"\x28\xb5\x2f\xfd"
+        mock_open.return_value.__enter__.open = mock_file_handle
+        images._handle_zstd_compression('path')
+        mock_move.assert_called_once_with('path', 'path.zstd')
+        mock_exec.assert_called_once_with('zstd', '-d', '--rm', 'path.zstd')
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(shutil, 'move', autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test__hanlde_zstd_compression_disabled(self, mock_open, mock_move,
+                                               mock_exec):
+        mock_file_handle = mock.Mock()
+        mock_file_handle.read.return_value = b"\x28\xb5\x2f\xfd"
+        mock_open.return_value.__enter__.open = mock_file_handle
+        CONF.set_override('disable_zstandard_decompression', True,
+                          group='conductor')
+        images._handle_zstd_compression('path')
+        mock_move.assert_not_called()
+        mock_exec.assert_not_called()
+
+
+class ImageDetectFileFormatTestCase(base.TestCase):
+
+    def setUp(self):
+        super().setUp()
+
+        read_patcher = mock.patch.object(
+            format_inspector.InspectWrapper, "read", autospec=True)
+        read_patcher.start()
+        self.addCleanup(read_patcher.stop)
+
+        formats_patcher = mock.patch.object(
+            format_inspector.InspectWrapper, "formats",
+            new=mock.PropertyMock(),
+        )
+        self.mock_formats = formats_patcher.start()
+        self.addCleanup(formats_patcher.stop)
+
+        format_patcher = mock.patch.object(
+            format_inspector.InspectWrapper, "format",
+            new=mock.PropertyMock(),
+        )
+        self.mock_format = format_patcher.start()
+        self.addCleanup(format_patcher.stop)
+
+        mock_file = mock.Mock()
+        mock_file.peek.return_value = True
+        open_patcher = mock.patch.object(builtins, 'open', autospec=True)
+        self.mock_open = open_patcher.start()
+        self.mock_open.return_value.__enter__.open = mock_file
+        self.addCleanup(open_patcher.stop)
+
+    def test_detect_file_format_passes(self):
+        self.mock_format.return_value = "spam"
+        self.mock_formats.side_effect = [None, [], ["spam"]]
+
+        self.assertEqual("spam", images.detect_file_format("foo"))
+        self.mock_open.assert_called_once_with("foo", "rb")
+        self.assertEqual(3, self.mock_formats.call_count)
+
+    def test_detect_file_format_fails_multiple(self):
+        self.mock_formats.return_value = ["spam", "ham"]
+        self.mock_format.side_effect = format_inspector.ImageFormatError()
+        self.assertRaises(format_inspector.ImageFormatError,
+                          images.detect_file_format, "foo")
+        self.mock_open.assert_called_once_with("foo", "rb")
+
+    def test_detect_file_format_passes_iso_gpt(self):
+        self.mock_formats.return_value = ["gpt", "iso"]
+        self.mock_format.side_effect = format_inspector.ImageFormatError()
+        self.assertEqual("iso", images.detect_file_format("foo"))
+        self.mock_open.assert_called_once_with("foo", "rb")
+
 
 class FsImageTestCase(base.TestCase):
 
@@ -658,8 +783,10 @@ class FsImageTestCase(base.TestCase):
     @mock.patch.object(utils, 'tempdir', autospec=True)
     @mock.patch.object(utils, 'write_to_file', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    @mock.patch.object(utils, 'dd', autospec=True)
     def test_create_vfat_image(
-            self, execute_mock, write_mock,
+            self, dd_mock, mkfs_mock, execute_mock, write_mock,
             tempdir_mock, create_root_fs_mock, os_listdir_mock):
 
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
@@ -672,37 +799,37 @@ class FsImageTestCase(base.TestCase):
         images.create_vfat_image('tgt_file', parameters=parameters,
                                  files_info=files_info, parameters_file='qwe',
                                  fs_size_kib=1000)
-
-        execute_mock.assert_has_calls([
-            mock.call('dd', 'if=/dev/zero', 'of=tgt_file', 'count=1',
-                      'bs=1000KiB'),
-            mock.call('mkfs', '-t', 'vfat', '-n', 'ir-vfd-dev', 'tgt_file'),
-            mock.call('mcopy', '-s', '/tempdir/b', '/tempdir/qwe', '-i',
-                      'tgt_file', '::')
-        ])
+        dd_mock.assert_called_once_with(
+            '/dev/zero', 'tgt_file', 'count=1', 'bs=1000KiB')
+        mkfs_mock.assert_called_once_with(
+            'vfat', 'tgt_file', 'ir-vfd-dev')
+        execute_mock.assert_called_once_with(
+            'mcopy', '-s', '/tempdir/b', '/tempdir/qwe', '-i',
+            'tgt_file', '::')
 
         parameters_file_path = os.path.join('/tempdir', 'qwe')
         write_mock.assert_called_once_with(parameters_file_path, 'p1=v1')
         create_root_fs_mock.assert_called_once_with('/tempdir', files_info)
         os_listdir_mock.assert_called_once_with('/tempdir')
 
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test_create_vfat_image_dd_fails(self, execute_mock):
+    @mock.patch.object(utils, 'dd', autospec=True)
+    def test_create_vfat_image_dd_fails(self, dd_mock):
 
-        execute_mock.side_effect = processutils.ProcessExecutionError
+        dd_mock.side_effect = processutils.ProcessExecutionError
         self.assertRaises(exception.ImageCreationFailed,
                           images.create_vfat_image, 'tgt_file')
 
     @mock.patch.object(utils, 'tempdir', autospec=True)
-    @mock.patch.object(utils, 'execute', autospec=True)
-    def test_create_vfat_image_mkfs_fails(self, execute_mock,
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    @mock.patch.object(utils, 'dd', autospec=True)
+    def test_create_vfat_image_mkfs_fails(self, dd_mock, mkfs_mock,
                                           tempdir_mock):
 
         mock_file_handle = mock.MagicMock(spec=io.BytesIO)
         mock_file_handle.__enter__.return_value = 'tempdir'
         tempdir_mock.return_value = mock_file_handle
 
-        execute_mock.side_effect = [None, processutils.ProcessExecutionError]
+        mkfs_mock.side_effect = processutils.ProcessExecutionError
         self.assertRaises(exception.ImageCreationFailed,
                           images.create_vfat_image, 'tgt_file')
 
