@@ -13,6 +13,7 @@
 import json
 import os
 from unittest import mock
+from unittest import skip
 
 from oslo_config import cfg
 from oslo_utils import uuidutils
@@ -40,26 +41,28 @@ class TestCommonFunctions(db_base.DbTestCase):
         self.client = mock.MagicMock()
 
     def _objects_setup(self, set_physnets):
-        pg1 = obj_utils.create_test_portgroup(
-            self.context, node_id=self.node.id)
-        pg1_ports = []
         # This portgroup contains 2 ports, both of them without VIF. The ports
         # are assigned to physnet physnet1.
         physical_network = 'physnet1' if set_physnets else None
+        pg1 = obj_utils.create_test_portgroup(
+            self.context, node_id=self.node.id,
+            physical_network=physical_network)
+        pg1_ports = []
         for i in range(2):
             pg1_ports.append(obj_utils.create_test_port(
                 self.context, node_id=self.node.id,
                 address='52:54:00:cf:2d:0%d' % i,
                 physical_network=physical_network,
                 uuid=uuidutils.generate_uuid(), portgroup_id=pg1.id))
-        pg2 = obj_utils.create_test_portgroup(
-            self.context, node_id=self.node.id, address='00:54:00:cf:2d:04',
-            name='foo2', uuid=uuidutils.generate_uuid())
-        pg2_ports = []
         # This portgroup contains 3 ports, one of them with 'some-vif'
         # attached, so the two free ones should be considered standalone.
         # The ports are assigned physnet physnet2.
         physical_network = 'physnet2' if set_physnets else None
+        pg2 = obj_utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address='00:54:00:cf:2d:04',
+            name='foo2', physical_network=physical_network,
+            uuid=uuidutils.generate_uuid())
+        pg2_ports = []
         for i in range(2, 4):
             pg2_ports.append(obj_utils.create_test_port(
                 self.context, node_id=self.node.id,
@@ -70,7 +73,7 @@ class TestCommonFunctions(db_base.DbTestCase):
             self.context, node_id=self.node.id,
             address='52:54:00:cf:2d:04',
             physical_network=physical_network,
-            internal_info={'tenant_vif_port_id': 'some-vif'},
+            internal_info={common.TENANT_VIF_KEY: 'some-vif'},
             uuid=uuidutils.generate_uuid(), portgroup_id=pg2.id))
         # This portgroup has 'some-vif-2' attached to it and contains one port,
         # so neither portgroup nor port can be considered free. The ports are
@@ -78,7 +81,8 @@ class TestCommonFunctions(db_base.DbTestCase):
         physical_network = 'physnet3' if set_physnets else None
         pg3 = obj_utils.create_test_portgroup(
             self.context, node_id=self.node.id, address='00:54:00:cf:2d:05',
-            name='foo3', uuid=uuidutils.generate_uuid(),
+            name='foo3', physical_network=physical_network,
+            uuid=uuidutils.generate_uuid(),
             internal_info={common.TENANT_VIF_KEY: 'some-vif-2'})
         pg3_ports = [obj_utils.create_test_port(
             self.context, node_id=self.node.id,
@@ -427,8 +431,11 @@ class TestCommonFunctions(db_base.DbTestCase):
                               task, self.vif_id, {'physnet2'})
 
     @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    def test_plug_port_to_tenant_network_client(self, mock_gc, mock_update):
+    def test_plug_port_to_tenant_network_client(self, mock_gc,
+                                                wait_mock_status,
+                                                mock_update):
         self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
         self.port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -438,8 +445,11 @@ class TestCommonFunctions(db_base.DbTestCase):
         self.assertTrue(mock_update.called)
 
     @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
-    def test_plug_port_to_tenant_network_no_client(self, mock_gc, mock_update):
+    def test_plug_port_to_tenant_network_no_client(self, mock_gc,
+                                                   wait_mock_status,
+                                                   mock_update):
         self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
         self.port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -479,9 +489,79 @@ class TestCommonFunctions(db_base.DbTestCase):
             wait_agent_mock.assert_called_once_with(
                 nclient, 'hostname')
             wait_port_mock.assert_called_once_with(
-                nclient, self.vif_id, 'ACTIVE')
+                nclient, self.vif_id, 'ACTIVE', fail_on_binding_failure=True)
             self.assertTrue(mock_update.called)
 
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    def test_plug_port_to_tenant_network_with_physical_network(
+            self, mock_gc, wait_mock_status, mock_update):
+        # Test that physical_network is included in binding:profile for port
+        nclient = mock.MagicMock()
+        mock_gc.return_value = nclient
+        self.port.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
+        self.port.physical_network = 'physnet1'
+        self.port.save()
+
+        expected_attrs = {
+            'binding:vnic_type': neutron_common.VNIC_BAREMETAL,
+            'binding:host_id': self.node.uuid,
+            'mac_address': self.port.address,
+            'binding:profile': {
+                'local_link_information': [self.port.local_link_connection],
+                'physical_network': 'physnet1'
+            }
+        }
+
+        with task_manager.acquire(self.context, self.node.id) as task:
+            common.plug_port_to_tenant_network(task, self.port)
+            mock_update.assert_called_once_with(
+                task.context, self.vif_id, expected_attrs)
+
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_port_status', autospec=True)
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    def test_plug_portgroup_to_tenant_network_with_physical_network(
+            self, mock_gc, wait_mock_status, mock_update):
+        # Test that physical_network is included in binding:profile for
+        # a portgroup
+        nclient = mock.MagicMock()
+        mock_gc.return_value = nclient
+        pg = obj_utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address='00:54:00:cf:2d:01',
+            physical_network='physnet1')
+        port1 = obj_utils.create_test_port(
+            self.context, node_id=self.node.id, address='52:54:00:cf:2d:01',
+            portgroup_id=pg.id, uuid=uuidutils.generate_uuid())
+        port2 = obj_utils.create_test_port(
+            self.context, node_id=self.node.id, address='52:54:00:cf:2d:02',
+            portgroup_id=pg.id, uuid=uuidutils.generate_uuid())
+        pg.internal_info = {common.TENANT_VIF_KEY: self.vif_id}
+        pg.save()
+
+        expected_attrs = {
+            'binding:vnic_type': neutron_common.VNIC_BAREMETAL,
+            'binding:host_id': self.node.uuid,
+            'mac_address': pg.address,
+            'binding:profile': {
+                'local_link_information': [port1.local_link_connection,
+                                           port2.local_link_connection],
+                'local_group_information': {
+                    'id': pg.uuid,
+                    'name': pg.name,
+                    'bond_mode': pg.mode,
+                    'bond_properties': {}
+                },
+                'physical_network': 'physnet1'
+            }
+        }
+
+        with task_manager.acquire(self.context, self.node.id) as task:
+            common.plug_port_to_tenant_network(task, pg)
+            mock_update.assert_called_once_with(
+                task.context, self.vif_id, expected_attrs)
+
 
 class TestVifPortIDMixin(db_base.DbTestCase):
 
@@ -710,6 +790,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         self.neutron_port = {'id': '132f871f-eaec-4fed-9475-0d54465e0f00',
                              'mac_address': '52:54:00:cf:2d:32'}
 
+    @mock.patch.object(neutron_common, 'update_neutron_port',
+                       autospec=True)
     @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
                        autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
@@ -718,7 +800,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach(self, mock_gpbpi, mock_upa, mock_client, mock_gfp,
-                        mock_save):
+                        mock_save, mock_update_port):
         vif = {'id': "fake_vif_id"}
         mock_gfp.return_value = self.port
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -726,6 +808,18 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mock_client.assert_called_once_with(context=task.context)
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
+            mock_update_port.assert_has_calls([
+                mock.call(None, 'fake_vif_id',
+                          {'binding:host_id': '', 'binding:profile': {}},
+                          None),
+                mock.call(None, 'fake_vif_id',
+                          {'mac_address': None}, None),
+                mock.call(task.context, mock.ANY,
+                          {'binding:vnic_type': 'baremetal',
+                           'binding:host_id': mock.ANY,
+                           'binding:profile': {}},
+                          client=None)
+            ])
         self.assertFalse(mock_gpbpi.called)
         mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
                                          {'id': 'fake_vif_id'})
@@ -749,6 +843,10 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                                          {'id': 'fake_vif_id'})
         self.assertFalse(mock_save.called)
 
+    @mock.patch.object(neutron_common, 'unbind_neutron_port_if_bound',
+                       autospec=True)
+    @mock.patch.object(neutron_common, 'update_neutron_port',
+                       autospec=True)
     @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
                        autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
@@ -757,7 +855,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_with_physnet(self, mock_gpbpi, mock_upa, mock_client,
-                                     mock_gfp, mock_save):
+                                     mock_gfp, mock_save, mock_port_update,
+                                     mock_unbind):
         self.port.physical_network = 'physnet1'
         self.port.save()
         vif = {'id': "fake_vif_id"}
@@ -766,14 +865,24 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_attach(task, vif)
             mock_client.assert_called_once_with(context=task.context)
+            mock_unbind.assert_called_once_with("fake_vif_id", client=mock.ANY,
+                                                context=task.context)
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
+            mock_port_update.assert_called_once_with(
+                task.context, mock.ANY,
+                {'binding:vnic_type': 'baremetal',
+                 'binding:host_id': mock.ANY,
+                 'binding:profile': {}},
+                client=None)
         mock_gpbpi.assert_called_once_with(mock_client.return_value,
                                            'fake_vif_id')
         mock_gfp.assert_called_once_with(task, 'fake_vif_id', {'physnet1'},
                                          {'id': 'fake_vif_id'})
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
 
+    @mock.patch.object(neutron_common, 'update_neutron_port',
+                       autospec=True)
     @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
                        autospec=True)
     @mock.patch.object(common, 'plug_port_to_tenant_network', autospec=True)
@@ -783,7 +892,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_active_node(self, mock_gpbpi, mock_upa, mock_client,
-                                    mock_gfp, mock_plug, mock_save):
+                                    mock_gfp, mock_plug, mock_save,
+                                    mock_port_update):
         self.node.provision_state = states.ACTIVE
         self.node.save()
         vif = {'id': "fake_vif_id"}
@@ -793,6 +903,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             mock_client.assert_called_once_with(context=task.context)
             mock_upa.assert_called_once_with(
                 "fake_vif_id", self.port.address, context=task.context)
+            mock_port_update.assert_not_called()
         self.assertFalse(mock_gpbpi.called)
         mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
                                          {'id': 'fake_vif_id'})
@@ -827,6 +938,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         mock_save.assert_called_once_with(self.port, "fake_vif_id")
         mock_plug.assert_called_once_with(task, self.port, mock.ANY)
 
+    @mock.patch.object(neutron_common, 'update_neutron_port',
+                       autospec=True)
     @mock.patch.object(common.VIFPortIDMixin, '_save_vif_to_port_like_obj',
                        autospec=True)
     @mock.patch.object(common, 'get_free_port_like_object', autospec=True)
@@ -835,7 +948,8 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_physnets_by_port_uuid',
                        autospec=True)
     def test_vif_attach_portgroup_no_address(self, mock_gpbpi, mock_upa,
-                                             mock_client, mock_gfp, mock_save):
+                                             mock_client, mock_gfp, mock_save,
+                                             mock_port_update):
         pg = obj_utils.create_test_portgroup(
             self.context, node_id=self.node.id, address=None)
         mock_gfp.return_value = pg
@@ -843,6 +957,19 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_attach(task, vif)
             mock_client.assert_called_once_with(context=task.context)
+            mock_port_update.assert_has_calls([
+                mock.call(None, 'fake_vif_id',
+                          {'binding:host_id': '', 'binding:profile': {}},
+                          None),
+                mock.call(None, 'fake_vif_id',
+                          {'mac_address': None}, None),
+                mock.call(task.context, mock.ANY,
+                          {'binding:vnic_type': 'baremetal',
+                           'binding:host_id': mock.ANY,
+                           'binding:profile': {}},
+                          client=None)
+            ])
+
         self.assertFalse(mock_gpbpi.called)
         mock_gfp.assert_called_once_with(task, 'fake_vif_id', set(),
                                          {'id': 'fake_vif_id'})
@@ -938,7 +1065,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_detach(task, 'fake_vif_id')
         mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
-        self.assertFalse(mock_unp.called)
+        self.assertTrue(mock_unp.called)
         mock_clear.assert_called_once_with(self.port)
 
     @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
@@ -953,7 +1080,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.vif_detach(task, 'fake_vif_id')
         mock_get.assert_called_once_with(self.interface, task, 'fake_vif_id')
-        self.assertFalse(mock_unp.called)
+        self.assertTrue(mock_unp.called)
         mock_clear.assert_called_once_with(pg)
 
     @mock.patch.object(common.VIFPortIDMixin, '_clear_vif_from_port_like_obj',
@@ -1057,6 +1184,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(mac_update_mock.called)
 
+    @skip("https://bugs.launchpad.net/ironic/+bug/2024994")
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
                 autospec=True)
     def test_port_changed_client_id(self, dhcp_update_mock):
@@ -1074,6 +1202,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                 mock.ANY, 'fake-id', expected_dhcp_opts, context=task.context)
         self.assertEqual(2, what_changed_mock.call_count)
 
+    @skip("https://bugs.launchpad.net/ironic/+bug/2024994")
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
                 autospec=True)
     def test_port_changed_extra_add_new_key(self, dhcp_update_mock):
@@ -1086,6 +1215,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(dhcp_update_mock.called)
 
+    @skip("https://bugs.launchpad.net/ironic/+bug/2024994")
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
                 autospec=True)
     def test_port_changed_client_id_fail(self, dhcp_update_mock):
@@ -1111,6 +1241,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
                               task, port)
         self.assertEqual(2, what_changed_mock.call_count)
 
+    @skip("https://bugs.launchpad.net/ironic/+bug/2024994")
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
                 autospec=True)
     def test_port_changed_client_id_no_vif_id(self, dhcp_update_mock):
@@ -1122,6 +1253,7 @@ class TestNeutronVifPortIDMixin(db_base.DbTestCase):
             self.interface.port_changed(task, self.port)
             self.assertFalse(dhcp_update_mock.called)
 
+    @skip("https://bugs.launchpad.net/ironic/+bug/2024994")
     @mock.patch('ironic.dhcp.neutron.NeutronDHCPApi.update_port_dhcp_opts',
                 autospec=True)
     def test_port_changed_message_format_failure(self, dhcp_update_mock):
